@@ -87,7 +87,11 @@ defmodule EExTest do
       expected = "123\n456\n789"
       assert_eval(expected, string, [], trim: true)
 
-      string = "<%= 123 %> \n  <%= 456 %>  \n  <%= 789 %>"
+      string = "<%= 123 %> \n\n456\n\n  <%= 789 %>"
+      expected = "123\n456\n789"
+      assert_eval(expected, string, [], trim: true)
+
+      string = "<%= 123 %> \n  \n456\n  \n  <%= 789 %>"
       expected = "123\n456\n789"
       assert_eval(expected, string, [], trim: true)
 
@@ -194,6 +198,24 @@ defmodule EExTest do
       assert_eval("foo baz", "foo <%= if false do %>bar<% else %>baz<% end %>")
     end
 
+    test "embedded code with comments in do end" do
+      assert_eval("foo bar", "foo <%= case true do %><%# comment %><% true -> %>bar<% end %>")
+
+      assert_eval(
+        "foo\n\nbar\n",
+        "foo\n<%= case true do %>\n<%# comment %>\n<% true -> %>\nbar\n<% end %>"
+      )
+    end
+
+    test "embedded code with multi-line comments in do end" do
+      assert_eval("foo bar", "foo <%= case true do %><%!-- comment --%><% true -> %>bar<% end %>")
+
+      assert_eval(
+        "foo\n\nbar\n",
+        "foo\n<%= case true do %>\n<%!-- comment --%>\n<% true -> %>\nbar\n<% end %>"
+      )
+    end
+
     test "embedded code with nested do end" do
       assert_eval("foo bar", "foo <%= if true do %><%= if true do %>bar<% end %><% end %>")
     end
@@ -211,6 +233,11 @@ defmodule EExTest do
         "<%= Enum.map([1, 2, 3], fn x -> %> <%= 100 + x %> <% end) %>"
       )
 
+      assert_eval(
+        " 101  102  103 ",
+        "<%= Enum.map([1, 2, 3], fn x ->\n%> <%= 100 + x %> <% end) %>"
+      )
+
       assert_eval(
         " 101  102  103 ",
         "<%= apply Enum, :map, [[1, 2, 3], fn x -> %> <%= 100 + x %> <% end] %>"
@@ -247,58 +274,192 @@ defmodule EExTest do
 
   describe "raises syntax errors" do
     test "when the token is invalid" do
-      assert_raise EEx.SyntaxError, "nofile:1:12: missing token '%>'", fn ->
+      message = """
+      nofile:1:5: expected closing '%>' for EEx expression
+        |
+      1 | foo <%= bar
+        |     ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
         EEx.compile_string("foo <%= bar")
       end
     end
 
     test "when middle expression is found without a start expression" do
-      assert_raise EEx.SyntaxError,
-                   "nofile:1:18: unexpected middle of expression <% else %>",
-                   fn ->
-                     EEx.compile_string("<% if true %> foo<% else %>bar<% end %>")
-                   end
+      message = """
+      nofile:5:1: unexpected middle of expression <% else %>
+        |
+      2 | <%= "content" %>
+      3 | <%= if true %>
+      4 |   <%= "foo" %>
+      5 | <% else %>
+        | ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string(
+          ~s(<h1>Hi!</h1>\n<%= "content" %>\n<%= if true %>\n  <%= "foo" %>\n<% else %>\n  bar<% end %>)
+        )
+      end
+    end
+
+    test "proper format line number of code snippet" do
+      message = """
+      nofile:11:1: unexpected middle of expression <% else %>
+         |
+       8 | <%= "content" %>
+       9 | <%= if true %>
+      10 |   <%= "foo" %>
+      11 | <% else %>
+         | ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string(
+          ~s(\n\n\n\n\n\n<h1>Hi!</h1>\n<%= "content" %>\n<%= if true %>\n  <%= "foo" %>\n<% else %>\n  bar<% end %>)
+        )
+      end
+    end
+
+    test "when there is only middle expression" do
+      message = """
+      nofile:1:1: unexpected middle of expression <% else %>
+        |
+      1 | <% else %>
+        | ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string(~s(<% else %>))
+      end
+    end
+
+    test "when it is missing a `do` in case expr" do
+      message = """
+      nofile:3:3: unexpected middle of expression <% :something -> %>
+        |
+      1 | content
+      2 | <%= case @var %>
+      3 |   <% :something -> %>
+        |   ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string("content\n<%= case @var %>\n  <% :something -> %>\n    bar<% end %>")
+      end
+    end
+
+    test "when it is a `do` in cond expr" do
+      message = """
+      nofile:3:3: unexpected middle of expression <% true -> %>
+        |
+      1 | content
+      2 | <%= cond %>
+      3 |   <% true -> %>
+        |   ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string("content\n<%= cond %>\n  <% true -> %>\n    bar<% end %>")
+      end
     end
 
     test "when end expression is found without a start expression" do
-      assert_raise EEx.SyntaxError, "nofile:1:5: unexpected end of expression <% end %>", fn ->
+      message = """
+      nofile:1:5: unexpected end of expression <% end %>
+        |
+      1 | foo <% end %>
+        |     ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
         EEx.compile_string("foo <% end %>")
       end
     end
 
     test "when start expression is found without an end expression" do
-      msg = "nofile:2:17: unexpected end of string, expected a closing '<% end %>'"
+      message = """
+      nofile:2:5: expected a closing '<% end %>' for block expression in EEx
+        |
+      1 | foo
+      2 | <%= if true do %>
+        |     ^\
+      """
 
-      assert_raise EEx.SyntaxError, msg, fn ->
-        EEx.compile_string("foo\n<% if true do %>")
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string("foo\n<%= if true do %>\nfoo\n")
+      end
+
+      message = """
+      nofile:3:3: expected a closing '<% end %>' for block expression in EEx
+        |
+      1 | foo
+      2 | <%=
+      3 |   if true do %>
+        |   ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string("foo\n<%=\n  if true do %>\nfoo\n", indentation: 0)
+      end
+
+      message = """
+      nofile:3:6: expected a closing '<% end %>' for block expression in EEx
+        |
+      1 |    foo
+      2 |    <%=
+      3 |      if true do %>
+        |      ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string("foo\n<%=\n  if true do %>\nfoo\n", indentation: 3)
+      end
+    end
+
+    test "when start expression with middle expression is found without an end expression" do
+      message = """
+      nofile:2:5: expected a closing '<% end %>' for block expression in EEx
+        |
+      1 | foo
+      2 | <%= if true do %>
+        |     ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string("foo\n<%= if true do %>\nfoo\n<% else %>\n")
+      end
+    end
+
+    test "when multiple start expressions is found without an end expression" do
+      message = """
+      nofile:5:5: expected a closing '<% end %>' for block expression in EEx
+        |
+      2 | <%= if true do %>
+      3 |   <%= @something %>
+      4 |\s
+      5 | <%= if @var do %>
+        |     ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string(
+          "foo\n<%= if true do %>\n  <%= @something %>\n\n<%= if @var do %>\nfoo\n"
+        )
       end
     end
 
     test "when nested end expression is found without a start expression" do
-      assert_raise EEx.SyntaxError, "nofile:1:30: unexpected end of expression <% end %>", fn ->
-        EEx.compile_string("foo <% if true do %><% end %><% end %>")
-      end
-    end
+      message = """
+      nofile:1:31: unexpected end of expression <% end %>
+        |
+      1 | foo <%= if true do %><% end %><% end %>
+        |                               ^\
+      """
 
-    test "when middle expression has a modifier" do
-      assert ExUnit.CaptureIO.capture_io(:stderr, fn ->
-               EEx.compile_string("foo <%= if true do %>true<%= else %>false<% end %>")
-             end) =~ ~s[unexpected beginning of EEx tag \"<%=\" on \"<%= else %>\"]
-    end
-
-    test "when end expression has a modifier" do
-      assert ExUnit.CaptureIO.capture_io(:stderr, fn ->
-               EEx.compile_string("foo <%= if true do %>true<% else %>false<%= end %>")
-             end) =~
-               ~s[unexpected beginning of EEx tag \"<%=\" on end of expression \"<%= end %>\"]
-    end
-
-    test "when trying to use marker '/' without implementation" do
-      msg =
-        ~r/unsupported EEx syntax <%\/ %> \(the syntax is valid but not supported by the current EEx engine\)/
-
-      assert_raise EEx.SyntaxError, msg, fn ->
-        EEx.compile_string("<%/ true %>")
+      assert_raise EEx.SyntaxError, message, fn ->
+        EEx.compile_string("foo <%= if true do %><% end %><% end %>")
       end
     end
 
@@ -310,22 +471,63 @@ defmodule EExTest do
         EEx.compile_string("<%| true %>")
       end
     end
-  end
 
-  describe "error messages" do
-    test "honor line numbers" do
-      assert_raise EEx.SyntaxError, "nofile:99:12: missing token '%>'", fn ->
-        EEx.compile_string("foo <%= bar", line: 99)
+    test "when trying to use marker '/' without implementation" do
+      msg =
+        ~r/unsupported EEx syntax <%\/ %> \(the syntax is valid but not supported by the current EEx engine\)/
+
+      assert_raise EEx.SyntaxError, msg, fn ->
+        EEx.compile_string("<%/ true %>")
       end
     end
 
+    test "honor line numbers" do
+      assert_raise EEx.SyntaxError,
+                   "nofile:100:6: expected closing '%>' for EEx expression",
+                   fn ->
+                     EEx.compile_string("foo\n bar <%= baz", line: 99)
+                   end
+    end
+
     test "honor file names" do
-      assert_raise EEx.SyntaxError, "my_file.eex:1:12: missing token '%>'", fn ->
+      message = """
+      my_file.eex:1:5: expected closing '%>' for EEx expression
+        |
+      1 | foo <%= bar
+        |     ^\
+      """
+
+      assert_raise EEx.SyntaxError, message, fn ->
         EEx.compile_string("foo <%= bar", file: "my_file.eex")
       end
     end
   end
 
+  describe "warnings" do
+    test "when middle expression has a modifier" do
+      assert ExUnit.CaptureIO.capture_io(:stderr, fn ->
+               EEx.compile_string("foo <%= if true do %>true<%= else %>false<% end %>")
+             end) =~ ~s[unexpected beginning of EEx tag \"<%=\" on \"<%= else %>\"]
+    end
+
+    test "when end expression has a modifier" do
+      assert ExUnit.CaptureIO.capture_io(:stderr, fn ->
+               EEx.compile_string("foo <%= if true do %>true<% else %>false<%= end %>")
+             end) =~
+               ~s[unexpected beginning of EEx tag \"<%=\" on \"<%= end %>\"]
+    end
+
+    test "from tokenizer" do
+      warning =
+        ExUnit.CaptureIO.capture_io(:stderr, fn ->
+          EEx.compile_string(~s'<%= :"foo" %>', file: "tokenizer.ex")
+        end)
+
+      assert warning =~ "found quoted atom \"foo\" but the quotes are not required"
+      assert warning =~ "tokenizer.ex:1:5"
+    end
+  end
+
   describe "environment" do
     test "respects line numbers" do
       expected = """
@@ -524,7 +726,7 @@ defmodule EExTest do
        <% "a" in y -> %>
         Good
        <% true -> %>
-        <% if true do %>true<% else %>false<% end %>
+        <%= if true do %>true<% else %>false<% end %>
         Bad
       <% end %>
       """
@@ -569,18 +771,6 @@ defmodule EExTest do
   end
 
   describe "buffers" do
-    test "unused buffers are kept out" do
-      string = """
-      <%= 123 %>
-      <% if true do %>
-        <%= 456 %>
-      <% end %>
-      <%= 789 %>
-      """
-
-      assert_eval("123\n\n789\n", string)
-    end
-
     test "inside comprehensions" do
       string = """
       <%= for _name <- packages || [] do %>
@@ -618,6 +808,22 @@ defmodule EExTest do
       assert EExTest.Compiled.__info__(:attributes)[:external_resource] ==
                [Path.join(__DIR__, "fixtures/eex_template_with_bindings.eex")]
     end
+
+    test "supports t:Path.t() paths" do
+      filename = to_charlist(Path.join(__DIR__, "fixtures/eex_template_with_bindings.eex"))
+      result = EEx.eval_file(filename, bar: 1)
+      assert_normalized_newline_equal("foo 1\n", result)
+    end
+
+    test "supports overriding file and line through options" do
+      filename = Path.join(__DIR__, "fixtures/eex_template_with_syntax_error.eex")
+
+      assert_raise EEx.SyntaxError,
+                   "my_file.eex:10:5: expected closing '%>' for EEx expression",
+                   fn ->
+                     EEx.eval_file(filename, _bindings = [], file: "my_file.eex", line: 10)
+                   end
+    end
   end
 
   describe "precompiled" do
@@ -640,7 +846,7 @@ defmodule EExTest do
                {21, {EExTest.Compiled, :after_compile, 0, [file: file, line: 21]}}
 
       assert EExTest.Compiled.unknown() ==
-               {26, {EExTest.Compiled, :unknown, 0, [file: 'unknown', line: 26]}}
+               {26, {EExTest.Compiled, :unknown, 0, [file: ~c"unknown", line: 26]}}
     end
   end
 
@@ -663,8 +869,8 @@ defmodule EExTest do
       buffer <> ":END"
     end
 
-    def handle_text(buffer, text) do
-      buffer <> ":TEXT(#{String.trim(text)})"
+    def handle_text(buffer, meta, text) do
+      buffer <> ":TEXT-#{meta[:line]}-#{meta[:column]}(#{String.trim(text)})"
     end
 
     def handle_expr(buffer, "/", expr) do
@@ -682,16 +888,16 @@ defmodule EExTest do
 
   describe "custom engines" do
     test "text" do
-      assert_eval("BODY(INIT:TEXT(foo))", "foo", [], engine: TestEngine)
+      assert_eval("BODY(INIT:TEXT-1-1(foo))", "foo", [], engine: TestEngine)
     end
 
     test "custom marker" do
-      assert_eval("BODY(INIT:TEXT(foo):DIV(:bar))", "foo <%/ :bar %>", [], engine: TestEngine)
+      assert_eval("BODY(INIT:TEXT-1-1(foo):DIV(:bar))", "foo <%/ :bar %>", [], engine: TestEngine)
     end
 
     test "begin/end" do
       assert_eval(
-        ~s[BODY(INIT:TEXT(foo):EQUAL(if do\n  "BEGIN:TEXT(this):END"\nelse\n  "BEGIN:TEXT(that):END"\nend))],
+        ~s[BODY(INIT:TEXT-1-1(foo):EQUAL(if do\n  "BEGIN:TEXT-1-17(this):END"\nelse\n  "BEGIN:TEXT-1-31(that):END"\nend))],
         "foo <%= if do %>this<% else %>that<% end %>",
         [],
         engine: TestEngine
@@ -708,6 +914,16 @@ defmodule EExTest do
     end
   end
 
+  describe "parser options" do
+    test "customizes parsed code" do
+      atoms_encoder = fn "not_jose", _ -> {:ok, :jose} end
+
+      assert_eval("valid", "<%= not_jose %>", [jose: "valid"],
+        parser_options: [static_atoms_encoder: atoms_encoder]
+      )
+    end
+  end
+
   defp assert_eval(expected, actual, binding \\ [], opts \\ []) do
     opts = Keyword.merge([file: __ENV__.file, engine: opts[:engine] || EEx.Engine], opts)
     result = EEx.eval_string(actual, binding, opts)
