@@ -49,11 +49,8 @@ defmodule Exception do
     %{general: message(exception), reason: "#" <> Atom.to_string(struct)}
   end
 
-  @doc """
-  Returns `true` if the given `term` is an exception.
-  """
-  # TODO: Deprecate this on Elixir v1.15
-  @doc deprecated: "Use Kernel.is_exception/1 instead"
+  @doc false
+  @deprecated "Use Kernel.is_exception/1 instead"
   def exception?(term)
   def exception?(%_{__exception__: true}), do: true
   def exception?(_), do: false
@@ -831,7 +828,7 @@ defmodule ArgumentError do
 
         not is_atom(function) ->
           "you attempted to apply a function named #{inspect(function)} on module #{inspect(module)}. " <>
-            "However #{inspect(function)} is not a valid function name. Function names (the second argument " <>
+            "However, #{inspect(function)} is not a valid function name. Function names (the second argument " <>
             "of apply) must always be an atom"
       end
 
@@ -951,7 +948,22 @@ defmodule CompileError do
 
   @impl true
   def message(%{file: file, line: line, description: description}) do
-    Exception.format_file_line(file && Path.relative_to_cwd(file), line) <> " " <> description
+    case Exception.format_file_line(file && Path.relative_to_cwd(file), line) do
+      "" -> description
+      formatted -> formatted <> " " <> description
+    end
+  end
+end
+
+defmodule Kernel.TypespecError do
+  defexception [:file, :line, :description]
+
+  @impl true
+  def message(%{file: file, line: line, description: description}) do
+    case Exception.format_file_line(file && Path.relative_to_cwd(file), line) do
+      "" -> description
+      formatted -> formatted <> " " <> description
+    end
   end
 end
 
@@ -1129,7 +1141,7 @@ defmodule UndefinedFunctionError do
   def hint_for_loaded_module(module, function, arity, exports) do
     cond do
       macro_exported?(module, function, arity) ->
-        ". However there is a macro with the same name and arity. " <>
+        ". However, there is a macro with the same name and arity. " <>
           "Be sure to require #{inspect(module)} if you intend to invoke this macro"
 
       message = otp_obsolete(module, function, arity) ->
@@ -1287,7 +1299,7 @@ defmodule FunctionClauseError do
     format_clause_fun = fn {args, guards} ->
       args = Enum.map_join(args, ", ", fun)
       base = "    #{kind} #{function}(#{args})"
-      Enum.reduce(guards, base, &"#{&2} when #{clause_to_string(&1, fun)}") <> "\n"
+      Enum.reduce(guards, base, &"#{&2} when #{clause_to_string(&1, fun, 0)}") <> "\n"
     end
 
     "\n\nThe following arguments were given to #{mfa}:\n" <>
@@ -1295,10 +1307,24 @@ defmodule FunctionClauseError do
       "#{format_clauses(clauses, format_clause_fun, @clause_limit)}"
   end
 
-  defp clause_to_string({op, _, [left, right]}, fun),
-    do: clause_to_string(left, fun) <> " #{op} " <> clause_to_string(right, fun)
+  defp clause_to_string({op, _, [left, right]} = node, fun, parent) do
+    case Code.Identifier.binary_op(op) do
+      {_side, precedence} ->
+        left = clause_to_string(left, fun, precedence)
+        right = clause_to_string(right, fun, precedence)
 
-  defp clause_to_string(node, fun), do: fun.(node)
+        if parent > precedence do
+          "(" <> left <> " #{op} " <> right <> ")"
+        else
+          left <> " #{op} " <> right
+        end
+
+      _ ->
+        fun.(node)
+    end
+  end
+
+  defp clause_to_string(node, fun, _precedence), do: fun.(node)
 
   defp format_args(args, inspect_fun) do
     args
@@ -1397,7 +1423,7 @@ defmodule KeyError do
     message = "key #{inspect(key)} not found"
 
     if term != nil do
-      message <> " in: #{inspect(term)}"
+      message <> " in: #{inspect(term, pretty: true, limit: :infinity)}"
     else
       message
     end
@@ -1639,7 +1665,7 @@ defmodule ErlangError do
 
   def normalize({:badkey, key, term}, _stacktrace) do
     message =
-      "key #{inspect(key)} not found in: #{inspect(term)}. " <>
+      "key #{inspect(key)} not found in: #{inspect(term, pretty: true, limit: :infinity)}\n\n" <>
         "If you are using the dot syntax, such as map.field, " <>
         "make sure the left-hand side of the dot is a map"
 
@@ -1650,7 +1676,8 @@ defmodule ErlangError do
     %CaseClauseError{term: term}
   end
 
-  def normalize({:with_clause, term}, _stacktrace) do
+  # :else_clause is aligned on what Erlang returns for `maybe`
+  def normalize({:else_clause, term}, _stacktrace) do
     %WithClauseError{term: term}
   end
 
