@@ -4,9 +4,8 @@ defmodule Inspect.Opts do
 
   The following fields are available:
 
-    * `:base` - prints integers as `:binary`, `:octal`, `:decimal`, or `:hex`,
-      defaults to `:decimal`. When inspecting binaries any `:base` other than
-      `:decimal` implies `binaries: :as_binaries`.
+    * `:base` - prints integers and binaries as `:binary`, `:octal`, `:decimal`,
+      or `:hex`. Defaults to `:decimal`.
 
     * `:binaries` - when `:as_binaries` all binaries will be printed in bit
       syntax.
@@ -14,9 +13,9 @@ defmodule Inspect.Opts do
       When `:as_strings` all binaries will be printed as strings, non-printable
       bytes will be escaped.
 
-      When the default `:infer`, the binary will be printed as a string if it
-      is printable, otherwise in bit syntax. See `String.printable?/1` to learn
-      when a string is printable.
+      When the default `:infer`, the binary will be printed as a string if `:base`
+      is `:decimal` and if it is printable, otherwise in bit syntax. See
+      `String.printable?/1` to learn when a string is printable.
 
     * `:charlists` - when `:as_charlists` all lists will be printed as charlists,
       non-printable elements will be escaped.
@@ -32,9 +31,10 @@ defmodule Inspect.Opts do
       to pass the custom options through.
 
       It supports some pre-defined keys:
-      - `:sort_maps` (since v1.15.0) - if set to `true`, sorts key-value pairs in maps.
-        This can be helpful to make map inspection deterministic for testing,
-        especially since key order is random since OTP 26.
+
+      - `:sort_maps` (since v1.14.4) - if set to `true`, sorts key-value pairs
+        in maps. This can be helpful to make map inspection deterministic for
+        testing, given maps key order is random.
 
     * `:inspect_fun` (since v1.9.0) - a function to build algebra documents.
       Defaults to `Inspect.Opts.default_inspect_fun/0`.
@@ -67,7 +67,9 @@ defmodule Inspect.Opts do
       colorized. The keys are types and the values are the colors to use for
       each type (for example, `[number: :red, atom: :blue]`). Types can include
       `:atom`, `:binary`, `:boolean`, `:list`, `:map`, `:number`, `:regex`,
-      `:string`, and `:tuple`. Custom data types may provide their own options.
+      `:string`, `:tuple`, or some types to represent AST like `:variable`,
+      `:call`, and `:operator`.
+      Custom data types may provide their own options.
       Colors can be any `t:IO.ANSI.ansidata/0` as accepted by `IO.ANSI.format/1`.
       A default list of colors can be retrieved from `IO.ANSI.syntax_colors/0`.
 
@@ -229,13 +231,13 @@ defmodule Inspect.Algebra do
 
   This implementation provides two types of breaks: `:strict` and `:flex`.
   When a group does not fit, all strict breaks are treated as newlines.
-  Flex breaks however are re-evaluated on every occurrence and may still
+  Flex breaks, however, are re-evaluated on every occurrence and may still
   be rendered flat. See `break/1` and `flex_break/1` for more information.
 
   This implementation also adds `force_unfit/1` and `next_break_fits/2` which
   give more control over the document fitting.
 
-    [0]: http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.34.2200
+    [0]: https://lindig.github.io/papers/strictly-pretty-2000.pdf
 
   """
 
