@@ -43,6 +43,13 @@ defmodule Protocol do
         def size(tuple), do: tuple_size(tuple)
       end
 
+  Finally, we can use the `Size` protocol to call the correct implementation:
+
+      Size.size({1, 2})
+      # => 2
+      Size.size(%{key: :value})
+      # => 1
+
   Note that we didn't implement it for lists as we don't have the
   `size` information on lists, rather its value needs to be
   computed with `length`.
@@ -449,7 +456,7 @@ defmodule Protocol do
   """
   @spec extract_protocols([charlist | String.t()]) :: [atom]
   def extract_protocols(paths) do
-    extract_matching_by_attribute(paths, 'Elixir.', fn module, attributes ->
+    extract_matching_by_attribute(paths, [?E, ?l, ?i, ?x, ?i, ?r, ?.], fn module, attributes ->
       case attributes[:__protocol__] do
         [fallback_to_any: _] -> module
         _ -> nil
@@ -478,7 +485,7 @@ defmodule Protocol do
   """
   @spec extract_impls(module, [charlist | String.t()]) :: [atom]
   def extract_impls(protocol, paths) when is_atom(protocol) do
-    prefix = Atom.to_charlist(protocol) ++ '.'
+    prefix = Atom.to_charlist(protocol) ++ [?.]
 
     extract_matching_by_attribute(paths, prefix, fn _mod, attributes ->
       case attributes[:__impl__] do
@@ -504,7 +511,7 @@ defmodule Protocol do
   end
 
   defp extract_from_file(path, file, prefix, callback) do
-    if :lists.prefix(prefix, file) and :filename.extension(file) == '.beam' do
+    if :lists.prefix(prefix, file) and :filename.extension(file) == [?., ?b, ?e, ?a, ?m] do
       extract_from_beam(:filename.join(path, file), callback)
     end
   end
@@ -548,7 +555,7 @@ defmodule Protocol do
 
   This function does not load the protocol at any point
   nor loads the new bytecode for the compiled module.
-  However each implementation must be available and
+  However, each implementation must be available and
   it will be loaded.
   """
   @spec consolidate(module, [module]) ::
@@ -565,7 +572,7 @@ defmodule Protocol do
   end
 
   defp beam_protocol(protocol) do
-    chunk_ids = [:debug_info, 'Docs', 'ExCk']
+    chunk_ids = [:debug_info, [?D, ?o, ?c, ?s], [?E, ?x, ?C, ?k]]
     opts = [:allow_missing_chunks]
 
     case :beam_lib.chunks(beam_file(protocol), chunk_ids, opts) do
@@ -895,6 +902,9 @@ defmodule Protocol do
       @compile {:inline, struct_impl_for: 1}
 
       unless Module.defines_type?(__MODULE__, {:t, 0}) do
+        @typedoc """
+        All the types that implement this protocol.
+        """
         @type t :: term
       end
 
