@@ -18,7 +18,7 @@ defmodule String do
       "hello world"
 
   The functions in this module act according to
-  [The Unicode Standard, Version 14.0.0](http://www.unicode.org/versions/Unicode14.0.0/).
+  [The Unicode Standard, Version 15.0.0](http://www.unicode.org/versions/Unicode15.0.0/).
 
   ## Interpolation
 
@@ -62,6 +62,7 @@ defmodule String do
     * `\\` - Single backslash
     * `\xNN` - A byte represented by the hexadecimal `NN`
     * `\uNNNN` - A Unicode code point represented by `NNNN`
+    * `\u{NNNNNN}` - A Unicode code point represented by `NNNNNN`
 
   Note it is generally not advised to use `\xNN` in Elixir
   strings, as introducing an invalid byte sequence would
@@ -285,11 +286,9 @@ defmodule String do
     * a compiled search pattern created by `:binary.compile_pattern/1`
 
   """
-  # TODO: Replace "nonempty_binary :: <<_::8, _::_*8>>" with "nonempty_binary()"
-  # when minimum requirement is >= OTP 24.
   @type pattern ::
           t()
-          | [nonempty_binary :: <<_::8, _::_*8>>]
+          | [nonempty_binary]
           | (compiled_search_pattern :: :binary.cp())
 
   @conditional_mappings [:greek, :turkic]
@@ -333,7 +332,7 @@ defmodule String do
     end
   end
 
-  for char <- '\n\r\t\v\b\f\e\d\a' do
+  for char <- [?\n, ?\r, ?\t, ?\v, ?\b, ?\f, ?\e, ?\d, ?\a] do
     defp recur_printable?(<<unquote(char), rest::binary>>, character_limit) do
       recur_printable?(rest, decrement(character_limit))
     end
@@ -674,7 +673,7 @@ defmodule String do
   defp do_split_at(string, position) do
     remaining = byte_size_remaining_at(string, position)
     start = byte_size(string) - remaining
-    <<left::size(start)-binary, right::size(remaining)-binary>> = string
+    <<left::size(^start)-binary, right::size(^remaining)-binary>> = string
     {left, right}
   end
 
@@ -1002,7 +1001,7 @@ defmodule String do
   defp replace_leading(string, match, replacement, prefix_size, suffix_size, acc)
        when suffix_size >= 0 do
     case string do
-      <<prefix::size(prefix_size)-binary, suffix::binary>> when prefix == match ->
+      <<prefix::size(^prefix_size)-binary, suffix::binary>> when prefix == match ->
         replace_leading(
           suffix,
           match,
@@ -1061,7 +1060,7 @@ defmodule String do
   defp replace_trailing(string, match, replacement, prefix_size, suffix_size, acc)
        when prefix_size >= 0 do
     case string do
-      <<prefix::size(prefix_size)-binary, suffix::binary>> when suffix == match ->
+      <<prefix::size(^prefix_size)-binary, suffix::binary>> when suffix == match ->
         replace_trailing(
           prefix,
           match,
@@ -1114,7 +1113,7 @@ defmodule String do
     prefix_size = byte_size(match)
 
     case string do
-      <<prefix::size(prefix_size)-binary, suffix::binary>> when prefix == match ->
+      <<prefix::size(^prefix_size)-binary, suffix::binary>> when prefix == match ->
         prepend_unless_empty(replacement, suffix)
 
       _ ->
@@ -1157,7 +1156,7 @@ defmodule String do
     prefix_size = byte_size(string) - suffix_size
 
     case string do
-      <<prefix::size(prefix_size)-binary, suffix::binary>> when suffix == match ->
+      <<prefix::size(^prefix_size)-binary, suffix::binary>> when suffix == match ->
         append_unless_empty(prefix, replacement)
 
       _ ->
@@ -1780,6 +1779,24 @@ defmodule String do
   @doc ~S"""
   Checks whether `string` contains only valid characters.
 
+  `algorithm` may be `:default` or `:fast_ascii`. Both algorithms are equivalent
+  from a validation perspective (they will always produce the same output), but
+  `:fast_ascii` can yield significant performance benefits in specific scenarios.
+
+  If all of the following conditions are true, you may want to experiment with
+  the `:fast_ascii` algorithm to see if it yields performance benefits in your
+  specific scenario:
+
+  * You are running Erlang/OTP 26 or newer on a 64 bit platform
+  * You expect most of your strings to be longer than ~64 bytes
+  * You expect most of your strings to contain mostly ASCII codepoints
+
+  Note that the `:fast_ascii` algorithm does not affect correctness, you can
+  expect the output of `String.valid?/2` to be the same regardless of algorithm.
+  The only difference to be expected is one of performance, which can be
+  expected to improve roughly linearly in string length compared to the
+  `:default` algorithm.
+
   ## Examples
 
       iex> String.valid?("a")
@@ -1797,17 +1814,32 @@ defmodule String do
       iex> String.valid?("asd" <> <<0xFFFF::16>>)
       false
 
+      iex> String.valid?("a", :fast_ascii)
+      true
+
+      iex> String.valid?(4)
+      ** (FunctionClauseError) no function clause matching in String.valid?/2
+
   """
   @spec valid?(t) :: boolean
-  def valid?(string)
+  def valid?(string, algorithm \\ :default)
 
-  def valid?(<<string::binary>>), do: valid_utf8?(string)
-  def valid?(_), do: false
+  def valid?(<<string::binary>>, :default), do: valid_utf8?(string)
+  def valid?(<<string::binary>>, :fast_ascii), do: valid_utf8_fast_ascii?(string)
 
   defp valid_utf8?(<<_::utf8, rest::bits>>), do: valid_utf8?(rest)
   defp valid_utf8?(<<>>), do: true
   defp valid_utf8?(_), do: false
 
+  defp valid_utf8_fast_ascii?(<<a::56, rest::bits>>)
+       when Bitwise.band(0x80808080808080, a) == 0 do
+    valid_utf8_fast_ascii?(rest)
+  end
+
+  defp valid_utf8_fast_ascii?(<<_::utf8, rest::bits>>), do: valid_utf8_fast_ascii?(rest)
+  defp valid_utf8_fast_ascii?(<<>>), do: true
+  defp valid_utf8_fast_ascii?(_), do: false
+
   @doc false
   @deprecated "Use String.valid?/1 instead"
   def valid_character?(string) do
@@ -2016,6 +2048,12 @@ defmodule String do
   @spec length(t) :: non_neg_integer
   def length(string) when is_binary(string), do: length(string, 0)
 
+  defp length(<<byte1, byte2, rest::binary>> = binary, acc)
+       when byte1 <= 127 and byte1 != ?\r and byte2 <= 127 and byte2 != ?\r do
+    skip = skip_length(rest, 1)
+    length(binary_part(binary, skip, byte_size(binary) - skip), acc + skip)
+  end
+
   defp length(gcs, acc) do
     case :unicode_util.gc(gcs) do
       [_ | rest] -> length(rest, acc + 1)
@@ -2024,6 +2062,13 @@ defmodule String do
     end
   end
 
+  defp skip_length(<<byte, rest::binary>>, acc)
+       when byte <= 127 and byte != ?\r,
+       do: skip_length(rest, acc + 1)
+
+  defp skip_length(_binary, acc),
+    do: acc
+
   @doc """
   Returns the grapheme at the `position` of the given UTF-8 `string`.
   If `position` is greater than `string` length, then it returns `nil`.
@@ -2439,7 +2484,9 @@ defmodule String do
   function will search if any of the strings in `contents`
   are part of `string`.
 
-  > Note: if you want to check if `string` is listed in `contents`,
+  > #### Searching for a string in a list {: .tip}
+  >
+  > If you want to check if `string` is listed in `contents`,
   > where `contents` is a list, use `Enum.member?(contents, string)`
   > instead.
 
@@ -2522,8 +2569,8 @@ defmodule String do
 
   ## Examples
 
-      iex> String.to_charlist("æß")
-      'æß'
+      iex> String.to_charlist("foo")
+      ~c"foo"
 
   """
   @spec to_charlist(t) :: charlist
@@ -2541,7 +2588,7 @@ defmodule String do
   end
 
   @doc """
-  Converts a string to an atom.
+  Converts a string to an existing atom or creates a new one.
 
   Warning: this function creates atoms dynamically and atoms are
   not garbage-collected. Therefore, `string` should not be an
@@ -2567,7 +2614,8 @@ defmodule String do
   end
 
   @doc """
-  Converts a string to an existing atom.
+  Converts a string to an existing atom or raises if
+  the atom does not exist.
 
   The maximum atom size is of 255 Unicode code points.
   Raises an `ArgumentError` if the atom does not exist.
@@ -2582,6 +2630,9 @@ defmodule String do
   > generally recommended to call `String.to_existing_atom/1` only to
   > convert atoms defined within the module making the function call
   > to `to_existing_atom/1`.
+  >
+  > To create a module name itself from a string safely,
+  > it is recommended to use `Module.safe_concat/1`.
 
   ## Examples
 
