@@ -158,7 +158,7 @@ translate({'try', Meta, [Opts]}, _Ann, S) ->
       {[], SC}
   end,
 
-  Else = elixir_erl_clauses:get_clauses(else, Opts, match),
+  Else = elixir_erl_clauses:get_clauses('else', Opts, match),
   {TElse, SE} = elixir_erl_clauses:clauses(Else, SA),
   {{'try', ?ann(Meta), unblock(TDo), TElse, TCatch, TAfter}, SE};
 
@@ -191,7 +191,7 @@ translate({with, Meta, [_ | _] = Args}, _Ann, S) ->
 
 %% Variables
 
-translate({'^', _, [{Name, VarMeta, Kind}]}, _Ann, #elixir_erl{context=match} = S) when is_atom(Name), is_atom(Kind) ->
+translate({'^', _, [{Name, VarMeta, Kind}]}, _Ann, S) when is_atom(Name), is_atom(Kind) ->
   {Var, VS} = elixir_erl_var:translate(VarMeta, Name, Kind, S),
 
   case S#elixir_erl.extra of
@@ -406,18 +406,18 @@ translate_with_else(Meta, [], S) ->
   {VarName, SC} = elixir_erl_var:build('_', S),
   Var = {var, Generated, VarName},
   {{clause, Generated, [Var], [], [Var]}, SC};
-translate_with_else(Meta, [{else, [{'->', _, [[{Var, VarMeta, Kind}], Clause]}]}], S) when is_atom(Var), is_atom(Kind) ->
+translate_with_else(Meta, [{'else', [{'->', _, [[{Var, VarMeta, Kind}], Clause]}]}], S) when is_atom(Var), is_atom(Kind) ->
   Ann = ?ann(Meta),
   Generated = erl_anno:set_generated(true, Ann),
   {ElseVarErl, SV} = elixir_erl_var:translate(VarMeta, Var, Kind, S#elixir_erl{context=match}),
   {TranslatedClause, SC} = translate(Clause, Ann, SV#elixir_erl{context=nil}),
   {{clause, Generated, [ElseVarErl], [], [TranslatedClause]}, SC};
-translate_with_else(Meta, [{else, Else}], S) ->
+translate_with_else(Meta, [{'else', Else}], S) ->
   Generated = ?generated(Meta),
   {ElseVarEx, ElseVarErl, SE} = elixir_erl_var:assign(Generated, S),
   {RaiseVar, _, SV} = elixir_erl_var:assign(Generated, SE),
 
-  RaiseExpr = {{'.', Generated, [erlang, error]}, Generated, [{with_clause, RaiseVar}]},
+  RaiseExpr = {{'.', Generated, [erlang, error]}, Generated, [{else_clause, RaiseVar}]},
   RaiseClause = {'->', Generated, [[RaiseVar], RaiseExpr]},
   GeneratedElse = [build_generated_clause(Generated, ElseClause) || ElseClause <- Else],
 
@@ -519,7 +519,7 @@ translate_bitstring(Meta, Args, S) ->
   build_bitstr(Args, ?ann(Meta), S, []).
 
 build_bitstr([{'::', Meta, [H, V]} | T], Ann, S, Acc) ->
-  {Size, Types} = extract_bit_info(V, Meta, S#elixir_erl{context=nil}),
+  {Size, Types} = extract_bit_info(V, Meta, S#elixir_erl{context=nil, extra=nil}),
   build_bitstr(T, Ann, S, Acc, H, Size, Types);
 build_bitstr([], Ann, S, Acc) ->
   {{bin, Ann, lists:reverse(Acc)}, S}.
@@ -560,7 +560,7 @@ extract_bit_type({'-', _, [L, R]}, Acc) ->
   extract_bit_type(L, extract_bit_type(R, Acc));
 extract_bit_type({unit, _, [Arg]}, Acc) ->
   [{unit, Arg} | Acc];
-extract_bit_type({Other, _, []}, Acc) ->
+extract_bit_type({Other, _, nil}, Acc) ->
   [Other | Acc].
 
 %% Optimizations that are specific to Erlang and change
