@@ -111,35 +111,37 @@ defmodule ExceptionTest do
   end
 
   test "format_stacktrace_entry/1 with file and line" do
-    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: 'file.ex', line: 10]}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: ~c"file.ex", line: 10]}) ==
              "file.ex:10: Foo.bar()"
 
-    assert Exception.format_stacktrace_entry({Foo, :bar, [1, 2, 3], [file: 'file.ex', line: 10]}) ==
+    assert Exception.format_stacktrace_entry(
+             {Foo, :bar, [1, 2, 3], [file: ~c"file.ex", line: 10]}
+           ) ==
              "file.ex:10: Foo.bar(1, 2, 3)"
 
-    assert Exception.format_stacktrace_entry({Foo, :bar, 1, [file: 'file.ex', line: 10]}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, 1, [file: ~c"file.ex", line: 10]}) ==
              "file.ex:10: Foo.bar/1"
   end
 
   test "format_stacktrace_entry/1 with file no line" do
-    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: 'file.ex']}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: ~c"file.ex"]}) ==
              "file.ex: Foo.bar()"
 
-    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: 'file.ex', line: 0]}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: ~c"file.ex", line: 0]}) ==
              "file.ex: Foo.bar()"
 
-    assert Exception.format_stacktrace_entry({Foo, :bar, [1, 2, 3], [file: 'file.ex']}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, [1, 2, 3], [file: ~c"file.ex"]}) ==
              "file.ex: Foo.bar(1, 2, 3)"
 
-    assert Exception.format_stacktrace_entry({Foo, :bar, 1, [file: 'file.ex']}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, 1, [file: ~c"file.ex"]}) ==
              "file.ex: Foo.bar/1"
   end
 
   test "format_stacktrace_entry/1 with application" do
-    assert Exception.format_stacktrace_entry({Exception, :bar, [], [file: 'file.ex']}) ==
+    assert Exception.format_stacktrace_entry({Exception, :bar, [], [file: ~c"file.ex"]}) ==
              "(elixir #{System.version()}) file.ex: Exception.bar()"
 
-    assert Exception.format_stacktrace_entry({Exception, :bar, [], [file: 'file.ex', line: 10]}) ==
+    assert Exception.format_stacktrace_entry({Exception, :bar, [], [file: ~c"file.ex", line: 10]}) ==
              "(elixir #{System.version()}) file.ex:10: Exception.bar()"
   end
 
@@ -419,6 +421,35 @@ defmodule ExceptionTest do
       assert Exception.blame(:exit, :function_clause, stack) == {:function_clause, stack}
     end
 
+    test "handles operators precedence" do
+      import PathHelpers
+
+      write_beam(
+        defmodule OperatorPrecedence do
+          def test!(x, y) when x in [1, 2, 3] and y >= 4, do: :ok
+        end
+      )
+
+      :code.delete(OperatorPrecedence)
+      :code.purge(OperatorPrecedence)
+
+      assert blame_message(OperatorPrecedence, & &1.test!(1, 2)) =~ """
+             no function clause matching in ExceptionTest.OperatorPrecedence.test!/2
+
+             The following arguments were given to ExceptionTest.OperatorPrecedence.test!/2:
+
+                 # 1
+                 1
+
+                 # 2
+                 2
+
+             Attempted function clauses (showing 1 out of 1):
+
+                 def test!(x, y) when (x === 1 or -x === 2- or -x === 3-) and -y >= 4-
+             """
+    end
+
     test "reverts is_struct macro on guards for blaming" do
       import PathHelpers
 
@@ -511,7 +542,7 @@ defmodule ExceptionTest do
                  "module.function(), make sure the left-hand side of the dot is a module atom"
 
       assert blame_message([], &apply(Kernel, &1, [1, 2])) ==
-               "you attempted to apply a function named [] on module Kernel. However [] is not a valid function name. " <>
+               "you attempted to apply a function named [] on module Kernel. However, [] is not a valid function name. " <>
                  "Function names (the second argument of apply) must always be an atom"
 
       assert blame_message(123, &apply(Kernel, :+, &1)) ==
@@ -566,7 +597,7 @@ defmodule ExceptionTest do
 
     test "annotates undefined function clause error with macro hints" do
       assert blame_message(Integer, & &1.is_odd(1)) ==
-               "function Integer.is_odd/1 is undefined or private. However there is " <>
+               "function Integer.is_odd/1 is undefined or private. However, there is " <>
                  "a macro with the same name and arity. Be sure to require Integer if " <>
                  "you intend to invoke this macro"
     end
@@ -612,12 +643,12 @@ defmodule ExceptionTest do
 
     test "annotates undefined key error with nil hints" do
       assert blame_message(nil, & &1.foo) ==
-               "key :foo not found in: nil. If you are using the dot syntax, " <>
+               "key :foo not found in: nil\n\nIf you are using the dot syntax, " <>
                  "such as map.field, make sure the left-hand side of the dot is a map"
 
       # we use `Code.eval_string/1` to escape the formatter and warnings
       assert blame_message("nil.foo", &Code.eval_string/1) ==
-               "key :foo not found in: nil. If you are using the dot syntax, " <>
+               "key :foo not found in: nil\n\nIf you are using the dot syntax, " <>
                  "such as map.field, make sure the left-hand side of the dot is a map"
     end
 
@@ -897,36 +928,34 @@ defmodule ExceptionTest do
     end
   end
 
-  if System.otp_release() >= "24" do
-    describe "error_info" do
-      test "badarg on erlang" do
-        assert message(:erlang, & &1.element("foo", "bar")) == """
-               errors were found at the given arguments:
+  describe "error_info" do
+    test "badarg on erlang" do
+      assert message(:erlang, & &1.element("foo", "bar")) == """
+             errors were found at the given arguments:
 
-                 * 1st argument: not an integer
-                 * 2nd argument: not a tuple
-               """
-      end
+               * 1st argument: not an integer
+               * 2nd argument: not a tuple
+             """
+    end
 
-      test "badarg on ets" do
-        ets = :ets.new(:foo, [])
-        :ets.delete(ets)
+    test "badarg on ets" do
+      ets = :ets.new(:foo, [])
+      :ets.delete(ets)
 
-        assert message(:ets, & &1.insert(ets, 1)) == """
-               errors were found at the given arguments:
+      assert message(:ets, & &1.insert(ets, 1)) == """
+             errors were found at the given arguments:
 
-                 * 1st argument: the table identifier does not refer to an existing ETS table
-                 * 2nd argument: not a tuple
-               """
-      end
+               * 1st argument: the table identifier does not refer to an existing ETS table
+               * 2nd argument: not a tuple
+             """
+    end
 
-      test "system_limit on counters" do
-        assert message(:counters, & &1.new(123_456_789_123_456_789_123_456_789, [])) == """
-               a system limit has been reached due to errors at the given arguments:
+    test "system_limit on counters" do
+      assert message(:counters, & &1.new(123_456_789_123_456_789_123_456_789, [])) == """
+             a system limit has been reached due to errors at the given arguments:
 
-                 * 1st argument: counters array size reached a system limit
-               """
-      end
+               * 1st argument: counters array size reached a system limit
+             """
     end
   end
 
