@@ -8,6 +8,21 @@ defmodule Kernel.WarningTest do
     capture_io(:stderr, fun)
   end
 
+  defmacro will_warn do
+    quote file: "demo" do
+      %{dup: 1, dup: 2}
+    end
+  end
+
+  test "warnings from macro" do
+    assert capture_err(fn ->
+             Code.eval_string("""
+             import Kernel.WarningTest
+             will_warn()
+             """)
+           end) =~ "key :dup will be overridden in map\n  demo:13"
+  end
+
   test "outdented heredoc" do
     output =
       capture_err(fn ->
@@ -26,6 +41,93 @@ defmodule Kernel.WarningTest do
     assert {:error, _} = Code.string_to_quoted(~s[:"foobar" do])
   end
 
+  describe "unicode identifier security" do
+    test "prevents Restricted codepoints in identifiers" do
+      exception = assert_raise SyntaxError, fn -> Code.string_to_quoted!("_shibㅤ = 1") end
+
+      assert Exception.message(exception) =~
+               "unexpected token: \"ㅤ\" (column 6, code point U+3164)"
+    end
+
+    test "warns on confusables" do
+      assert capture_err(fn -> Code.string_to_quoted("а=1; a=1") end) =~
+               "confusable identifier: 'a' looks like 'а' on line 1"
+
+      assert capture_err(fn -> Code.string_to_quoted("[{:а, 1}, {:a, 1}]") end) =~
+               "confusable identifier: 'a' looks like 'а' on line 1"
+
+      assert capture_err(fn -> Code.string_to_quoted("[а: 1, a: 1]") end) =~
+               "confusable identifier: 'a' looks like 'а' on line 1"
+
+      assert capture_err(fn -> Code.string_to_quoted("quote do: [а(1), a(1)]") end) =~
+               "confusable identifier: 'a' looks like 'а' on line 1"
+
+      assert capture_err(fn -> Code.string_to_quoted("力=1; カ=1") end) =~
+               "confusable identifier: 'カ' looks like '力' on line 1"
+
+      # by convention, doesn't warn on ascii-only confusables
+      assert capture_err(fn -> Code.string_to_quoted("x0 = xO = 1") end) == ""
+      assert capture_err(fn -> Code.string_to_quoted("l1 = ll = 1") end) == ""
+
+      # works with a custom atom encoder
+      assert capture_err(fn ->
+               Code.string_to_quoted("[{:а, 1}, {:a, 1}]",
+                 static_atoms_encoder: fn token, _ -> {:ok, {:wrapped, token}} end
+               )
+             end) =~
+               "confusable identifier: 'a' looks like 'а' on line 1"
+    end
+
+    test "prevents unsafe script mixing in identifiers" do
+      exception =
+        assert_raise SyntaxError, fn ->
+          Code.string_to_quoted!("if аdmin_, do: :ok, else: :err")
+        end
+
+      assert Exception.message(exception) =~
+               "nofile:1:9: invalid mixed-script identifier found: аdmin"
+
+      assert Exception.message(exception) =~ """
+               \\u0430 а {Cyrillic}
+               \\u0064 d {Latin}
+               \\u006D m {Latin}
+               \\u0069 i {Latin}
+               \\u006E n {Latin}
+               \\u005F _
+             """
+
+      # includes suggestion about what to change
+      assert Exception.message(exception) =~ """
+             Hint: You could write the above in a similar way that is accepted by Elixir:
+
+                 "admin_" (code points 0x00061 0x00064 0x0006D 0x00069 0x0006E 0x0005F)
+             """
+
+      # a is in cyrillic
+      assert_raise SyntaxError, ~r/mixed/, fn -> Code.string_to_quoted!("[аdmin: 1]") end
+      assert_raise SyntaxError, ~r/mixed/, fn -> Code.string_to_quoted!("[{:аdmin, 1}]") end
+      assert_raise SyntaxError, ~r/mixed/, fn -> Code.string_to_quoted!("quote do: аdmin(1)") end
+      assert_raise SyntaxError, ~r/mixed/, fn -> Code.string_to_quoted!("рос_api = 1") end
+
+      # T is in cyrillic
+      assert_raise SyntaxError, ~r/mixed/, fn -> Code.string_to_quoted!("[Тシャツ: 1]") end
+    end
+
+    test "allows legitimate script mixing" do
+      # writing systems that legitimately mix multiple scripts, and Common chars like _
+      assert capture_err(fn -> Code.eval_string("幻ㄒㄧㄤ = 1") end) == ""
+      assert capture_err(fn -> Code.eval_string("幻ㄒㄧㄤ1 = 1") end) == ""
+      assert capture_err(fn -> Code.eval_string("__सवव_1? = 1") end) == ""
+
+      # uts39 5.2 allowed 'highly restrictive' script mixing, like 't-shirt' in Jpan:
+      assert capture_err(fn -> Code.string_to_quoted!(":Tシャツ") end) == ""
+
+      # elixir's normalizations combine scriptsets of the 'from' and 'to' characters,
+      # ex: {Common} MICRO => {Greek} MU == {Common, Greek}; Common intersects w/all
+      assert capture_err(fn -> Code.string_to_quoted!("μs") end) == ""
+    end
+  end
+
   test "operators formed by many of the same character followed by that character" do
     output =
       capture_err(fn ->
@@ -50,6 +152,7 @@ defmodule Kernel.WarningTest do
       assert capture_err(fn -> Code.eval_string(~s/:"Foo"/) end) == ""
       assert capture_err(fn -> Code.eval_string(~s/:"foo@bar"/) end) == ""
       assert capture_err(fn -> Code.eval_string(~s/:"héllò"/) end) == ""
+      assert capture_err(fn -> Code.eval_string(~s/:"3L1X1R"/) end) == ""
     end
 
     test "warns for unnecessary quotes" do
@@ -64,6 +167,11 @@ defmodule Kernel.WarningTest do
     end
   end
 
+  test "warns on :: as atom" do
+    assert capture_err(fn -> Code.eval_string(~s/:::/) end) =~
+             "atom ::: must be written between quotes, as in :\"::\", to avoid ambiguity"
+  end
+
   test "unused variable" do
     output =
       capture_err(fn ->
@@ -86,6 +194,35 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
+  test "unused variable that could be pinned" do
+    output =
+      capture_err(fn ->
+        # Note we use compile_string because eval_string does not emit unused vars warning
+        Code.compile_string("""
+        defmodule Sample do
+          def test do
+            compare_local = "hello"
+            match?(compare_local, "hello")
+
+            compare_nested = "hello"
+            case "hello" do
+              compare_nested -> true
+              _other -> false
+            end
+          end
+        end
+        """)
+      end)
+
+    assert output =~
+             "variable \"compare_local\" is unused (there is a variable with the same name in the context, use the pin operator (^) to match on it or prefix this variable with underscore if it is not meant to be used)"
+
+    assert output =~
+             "variable \"compare_nested\" is unused (there is a variable with the same name in the context, use the pin operator (^) to match on it or prefix this variable with underscore if it is not meant to be used)"
+  after
+    purge(Sample)
+  end
+
   test "unused compiler variable" do
     output =
       capture_err(fn ->
@@ -104,113 +241,73 @@ defmodule Kernel.WarningTest do
   end
 
   test "nested unused variable" do
-    message = "variable \"x\" is unused"
+    messages = ["undefined variable \"x\"", "variable \"x\" is unused"]
 
-    assert capture_err(fn ->
-             assert_raise CompileError, ~r/undefined function x/, fn ->
-               Code.eval_string("""
-               case false do
-                 true -> x = 1
-                 _ -> 1
-               end
-               x
-               """)
-             end
-           end) =~ message
+    assert_compile_error(messages, """
+    case false do
+      true -> x = 1
+      _ -> 1
+    end
+    x
+    """)
 
-    assert capture_err(fn ->
-             assert_raise CompileError, ~r/undefined function x/, fn ->
-               Code.eval_string("""
-               false and (x = 1)
-               x
-               """)
-             end
-           end) =~ message
+    assert_compile_error(messages, """
+    false and (x = 1)
+    x
+    """)
 
-    assert capture_err(fn ->
-             assert_raise CompileError, ~r/undefined function x/, fn ->
-               Code.eval_string("""
-               true or (x = 1)
-               x
-               """)
-             end
-           end) =~ message
+    assert_compile_error(messages, """
+    true or (x = 1)
+    x
+    """)
 
-    assert capture_err(fn ->
-             assert_raise CompileError, ~r/undefined function x/, fn ->
-               Code.eval_string("""
-               if false do
-                 x = 1
-               end
-               x
-               """)
-             end
-           end) =~ message
+    assert_compile_error(messages, """
+    if false do
+      x = 1
+    end
+    x
+    """)
 
-    assert capture_err(fn ->
-             assert_raise CompileError, ~r/undefined function x/, fn ->
-               Code.eval_string("""
-               cond do
-                 false -> x = 1
-                 true -> 1
-               end
-               x
-               """)
-             end
-           end) =~ message
+    assert_compile_error(messages, """
+    cond do
+      false -> x = 1
+      true -> 1
+    end
+    x
+    """)
 
-    assert capture_err(fn ->
-             assert_raise CompileError, ~r/undefined function x/, fn ->
-               Code.eval_string("""
-               receive do
-                 :foo -> x = 1
-               after
-                 0 -> 1
-               end
-               x
-               """)
-             end
-           end) =~ message
+    assert_compile_error(messages, """
+    receive do
+      :foo -> x = 1
+    after
+      0 -> 1
+    end
+    x
+    """)
 
-    assert capture_err(fn ->
-             assert_raise CompileError, ~r/undefined function x/, fn ->
-               Code.eval_string("""
-               false && (x = 1)
-               x
-               """)
-             end
-           end) =~ message
+    assert_compile_error(messages, """
+    false && (x = 1)
+    x
+    """)
 
-    assert capture_err(fn ->
-             assert_raise CompileError, ~r/undefined function x/, fn ->
-               Code.eval_string("""
-               true || (x = 1)
-               x
-               """)
-             end
-           end) =~ message
+    assert_compile_error(messages, """
+    true || (x = 1)
+    x
+    """)
 
-    assert capture_err(fn ->
-             assert_raise CompileError, ~r/undefined function x/, fn ->
-               Code.eval_string("""
-               with true <- true do
-                 x = false
-               end
-               x
-               """)
-             end
-           end) =~ message
+    assert_compile_error(messages, """
+    with true <- true do
+      x = false
+    end
+    x
+    """)
 
-    assert capture_err(fn ->
-             assert_raise CompileError, ~r/undefined function x/, fn ->
-               Code.eval_string("""
-               fn ->
-                 x = true
-               end
-               x
-               """)
-             end
-           end) =~ message
+    assert_compile_error(messages, """
+    fn ->
+      x = true
+    end
+    x
+    """)
   end
 
   test "unused variable in redefined function in different file" do
@@ -466,17 +563,17 @@ defmodule Kernel.WarningTest do
              end
              """)
            end) =~
-             "the default values for the first 2 optional arguments in b/3 are never used\n  nofile:3"
+             "the default value for the last optional argument in b/3 is never used\n  nofile:3"
 
     assert capture_err(fn ->
              Code.eval_string(~S"""
              defmodule Sample3 do
-               def a, do: b(1)
-               defp b(arg1 \\ 1, arg2 \\ 2, arg3 \\ 3), do: [arg1, arg2, arg3]
+               def a, do: b(1, 2)
+               defp b(arg1, arg2 \\ 2, arg3 \\ 3, arg4 \\ 4), do: [arg1, arg2, arg3, arg4]
              end
              """)
            end) =~
-             "the default value for the first optional argument in b/3 is never used\n  nofile:3"
+             "the default values for the last 2 optional arguments in b/4 are never used"
 
     assert capture_err(fn ->
              Code.eval_string(~S"""
@@ -490,17 +587,6 @@ defmodule Kernel.WarningTest do
     assert capture_err(fn ->
              Code.eval_string(~S"""
              defmodule Sample5 do
-               def a, do: b(1, 2, 3)
-               defp b(arg1 \\ 1, arg2 \\ 2, arg3 \\ 3)
-
-               defp b(arg1, arg2, arg3), do: [arg1, arg2, arg3]
-             end
-             """)
-           end) =~ "default values for the optional arguments in b/3 are never used\n  nofile:3"
-
-    assert capture_err(fn ->
-             Code.eval_string(~S"""
-             defmodule Sample6 do
                def a, do: b(1, 2)
                defp b(arg1 \\ 1, arg2 \\ 2, arg3 \\ 3)
 
@@ -508,9 +594,9 @@ defmodule Kernel.WarningTest do
              end
              """)
            end) =~
-             "the default values for the first 2 optional arguments in b/3 are never used\n  nofile:3"
+             "the default value for the last optional argument in b/3 is never used\n  nofile:3"
   after
-    purge([Sample1, Sample2, Sample3, Sample4, Sample5, Sample6])
+    purge([Sample1, Sample2, Sample3, Sample4, Sample5])
   end
 
   test "unused import" do
@@ -532,6 +618,14 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
+  test "unknown import" do
+    assert capture_err(fn ->
+             Code.compile_string("""
+             import(Kernel, only: [invalid: 1])
+             """)
+           end) =~ "cannot import Kernel.invalid/1 because it is undefined or private"
+  end
+
   test "unused import of one of the functions in :only" do
     output =
       capture_err(fn ->
@@ -562,6 +656,24 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
+  test "duplicated function on import options" do
+    assert capture_err(fn ->
+             Code.compile_string("""
+             defmodule Kernel.WarningsTest.DuplicatedFunctionOnImportOnly do
+               import List, only: [wrap: 1, keyfind: 3, wrap: 1]
+             end
+             """)
+           end) =~ "invalid :only option for import, wrap/1 is duplicated\n"
+
+    assert capture_err(fn ->
+             Code.compile_string("""
+             defmodule Kernel.WarningsTest.DuplicatedFunctionOnImportExcept do
+               import List, except: [wrap: 1, keyfind: 3, wrap: 1]
+             end
+             """)
+           end) =~ "invalid :except option for import, wrap/1 is duplicated"
+  end
+
   test "unused alias" do
     assert capture_err(fn ->
              Code.compile_string("""
@@ -850,6 +962,20 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
+  test "parens with module attribute" do
+    assert capture_err(fn ->
+             Code.eval_string("""
+             defmodule Sample do
+               @foo 13
+               @foo()
+             end
+             """)
+           end) =~
+             "the @foo() notation (with parentheses) is deprecated, please use @foo (without parentheses) instead"
+  after
+    purge(Sample)
+  end
+
   test "undefined module attribute in function" do
     assert capture_err(fn ->
              Code.eval_string("""
@@ -938,7 +1064,7 @@ defmodule Kernel.WarningTest do
                def foo, do: Atom.to_string "abc"
              end
              """)
-           end) =~ ~r"this expression will fail with ArgumentError\n.*nofile:2"
+           end) =~ "the call to Atom.to_string/1 will fail with ArgumentError\n  nofile:2"
 
     assert capture_err(fn ->
              Code.eval_string("""
@@ -946,7 +1072,7 @@ defmodule Kernel.WarningTest do
                def foo, do: 1 + nil
              end
              """)
-           end) =~ ~r"this expression will fail with ArithmeticError\n.*nofile:2"
+           end) =~ "the call to +/2 will fail with ArithmeticError\n  nofile:2"
   after
     purge([Sample1, Sample2])
   end
@@ -1070,18 +1196,6 @@ defmodule Kernel.WarningTest do
     purge(EmptyBehaviour)
   end
 
-  test "undefined behavior" do
-    assert capture_err(fn ->
-             Code.eval_string("""
-             defmodule Sample do
-               @behavior Hello
-             end
-             """)
-           end) =~ "@behavior attribute is not supported, please use @behaviour instead"
-  after
-    purge(Sample)
-  end
-
   test "undefined function for protocol" do
     assert capture_err(fn ->
              Code.eval_string("""
@@ -1145,35 +1259,24 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
-  test "with and do clauses emit errors, else clauses do not" do
+  test "warning on unnecessary code point escape" do
     assert capture_err(fn ->
-             Code.compile_string("""
-             with {:first, int} when is_integer(int) <- {:second, Integer.gcd(2, 4)} do
-               int
-             end
-             """)
-           end) =~ "this clause cannot match"
+             Code.eval_string("?\\n + ?\\\\")
+           end) == ""
 
     assert capture_err(fn ->
-             Code.compile_string("""
-             with {:first, int1} when is_integer(int1) <- {:first, Integer.gcd(2, 4)},
-                  {:second, int2} when is_integer(int2) <- {:second, Integer.gcd(2, 4)} do
-               {:ok, int1 + int2}
-             else
-               {:first, nil} -> {:error, "first number is not integer"}
-               {:second, nil} -> {:error, "second number is not integer"}
-             end
-             """)
-           end) == ""
-  after
-    purge(Sample1)
-    purge(Sample2)
+             Code.eval_string("?\\w")
+           end) =~ "unknown escape sequence ?\\w, use ?w instead"
   end
 
   test "warning on code point escape" do
     assert capture_err(fn ->
              Code.eval_string("? ")
            end) =~ "found ? followed by code point 0x20 (space), please use ?\\s instead"
+
+    assert capture_err(fn ->
+             Code.eval_string("?\\ ")
+           end) =~ "found ?\\ followed by code point 0x20 (space), please use ?\\s instead"
   end
 
   test "duplicated docs in the same clause" do
@@ -1403,70 +1506,56 @@ defmodule Kernel.WarningTest do
 
       assert capture_err(fn ->
                Code.eval_string("""
-               defmodule Sample do
+               defmodule Sample1 do
                  @type my_type :: ann_type :: nested_ann_type :: atom
                end
                """)
              end) =~ message
 
-      purge(Sample)
+      purge(Sample1)
 
       assert capture_err(fn ->
                Code.eval_string("""
-               defmodule Sample do
+               defmodule Sample2 do
                  @type my_type :: ann_type :: nested_ann_type :: atom | port
                end
                """)
              end) =~ message
 
-      purge(Sample)
+      purge(Sample2)
 
       assert capture_err(fn ->
                Code.eval_string("""
-               defmodule Sample do
+               defmodule Sample3 do
                  @spec foo :: {pid, ann_type :: nested_ann_type :: atom}
                  def foo, do: nil
                end
                """)
              end) =~ message
     after
-      purge(Sample)
+      purge([Sample1, Sample2, Sample3])
     end
 
     test "invalid type annotations" do
-      message =
-        "invalid type annotation. When using the | operator to represent the union of types, " <>
-          "make sure to wrap type annotations in parentheses"
+      assert capture_err(fn ->
+               Code.eval_string("""
+               defmodule Sample1 do
+                 @type my_type :: (pid() :: atom)
+               end
+               """)
+             end) =~ "invalid type annotation. The left side of :: must be a variable, got: pid()"
 
       assert capture_err(fn ->
                Code.eval_string("""
-               defmodule Sample do
+               defmodule Sample2 do
                  @type my_type :: pid | ann_type :: atom
                end
                """)
-             end) =~ message
-
-      purge(Sample)
-
-      assert capture_err(fn ->
-               Code.eval_string("""
-               defmodule Sample do
-                 @type my_type :: pid | ann_type :: atom | port
-               end
-               """)
-             end) =~ message
-
-      purge(Sample)
-
-      assert capture_err(fn ->
-               Code.eval_string("""
-               defmodule Sample do
-                 @type my_type :: {port, pid | ann_type :: atom | port}
-               end
-               """)
-             end) =~ message
+             end) =~
+               "invalid type annotation. The left side of :: must be a variable, got: pid | ann_type. " <>
+                 "Note \"left | right :: ann\" is the same as \"(left | right) :: ann\""
     after
-      purge(Sample)
+      purge([Sample1, Sample2])
     end
   end
 
@@ -1537,7 +1626,30 @@ defmodule Kernel.WarningTest do
            end) =~ "parentheses are required when piping into a function call"
   end
 
-  test "variable is being expanded to function call" do
+  test "keywords without explicit parentheses" do
+    assert capture_err(fn ->
+             Code.eval_string("""
+             quote do
+               IO.inspect arg, label: if true, do: "foo", else: "baz"
+             end
+             """)
+           end) =~ "missing parentheses for expression following \"label:\" keyword. "
+  end
+
+  test "do+end with operator without explicit parentheses" do
+    assert capture_err(fn ->
+             Code.eval_string("""
+             quote do
+               case do
+               end || raise 1, 2
+             end
+             """)
+           end) =~ "missing parentheses on expression following operator \"||\""
+  end
+
+  test "variable is being expanded to function call (on_undefined_variable: warn)" do
+    Code.put_compiler_option(:on_undefined_variable, :warn)
+
     output =
       capture_err(fn ->
         Code.eval_string("""
@@ -1551,6 +1663,7 @@ defmodule Kernel.WarningTest do
     assert output =~ "variable \"self\" does not exist and is being expanded to \"self()\""
     assert output =~ "variable \"node\" does not exist and is being expanded to \"node()\""
   after
+    Code.put_compiler_option(:on_undefined_variable, :raise)
     purge(Sample)
   end
 
@@ -1617,11 +1730,6 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
-  test "System.stacktrace is deprecated outside catch/rescue" do
-    assert capture_err(fn -> Code.eval_string("System.stacktrace()") end) =~
-             "System.stacktrace/0 is deprecated"
-  end
-
   test "unused variable in defguard" do
     assert capture_err(fn ->
              Code.eval_string("""
@@ -1813,6 +1921,11 @@ defmodule Kernel.WarningTest do
     end
   end
 
+  test "warnings on trailing comma on call" do
+    assert capture_err(fn -> Code.eval_string("Keyword.merge([], foo: 1,)") end) =~
+             "trailing commas are not allowed inside function/macro call arguments"
+  end
+
   test "defstruct warns with duplicate keys" do
     assert capture_err(fn ->
              Code.eval_string("""
@@ -1841,6 +1954,17 @@ defmodule Kernel.WarningTest do
              "extra parentheses on a remote function capture &System.pid()/0 have been deprecated. Please remove the parentheses: &System.pid/0"
   end
 
+  defp assert_compile_error(messages, string) do
+    captured =
+      capture_err(fn ->
+        assert_raise CompileError, fn -> Code.eval_string(string) end
+      end)
+
+    for message <- List.wrap(messages) do
+      assert captured =~ message
+    end
+  end
+
   defp purge(list) when is_list(list) do
     Enum.each(list, &purge/1)
   end
