@@ -21,6 +21,12 @@ defmodule KernelTest do
     end
   end
 
+  test "op ambiguity" do
+    max = 1
+    assert max == 1
+    assert max(1, 2) == 2
+  end
+
   describe "=/2" do
     test "can be reassigned" do
       var = 1
@@ -69,8 +75,8 @@ defmodule KernelTest do
       [1, 2] ++ var = [1, 2, 3]
       assert var == [3]
 
-      'ab' ++ var = 'abc'
-      assert var == 'c'
+      ~c"ab" ++ var = ~c"abc"
+      assert var == ~c"c"
 
       [:a, :b] ++ var = [:a, :b, :c]
       assert var == [:c]
@@ -208,6 +214,7 @@ defmodule KernelTest do
 
     user = struct(User, name: "meg")
     assert user == %User{name: "meg"}
+    assert struct(user, %{name: "meg"}) == user
 
     assert struct(user, unknown: "key") == user
     assert struct(user, %{name: "john"}) == %User{name: "john"}
@@ -417,6 +424,14 @@ defmodule KernelTest do
     assert exception_or_map?(%RuntimeError{}, RuntimeError) == true
   end
 
+  test "then/2" do
+    assert 1 |> then(fn x -> x * 2 end) == 2
+
+    assert_raise BadArityError, fn ->
+      1 |> then(fn x, y -> x * y end)
+    end
+  end
+
   test "if/2 boolean optimization does not leak variables during expansion" do
     if false do
       :ok
@@ -425,6 +440,12 @@ defmodule KernelTest do
     end
   end
 
+  describe ".." do
+    test "returns 0..-1//1" do
+      assert (..) == 0..-1//1
+    end
+  end
+
   describe "in/2" do
     test "too large list in guards" do
       defmodule TooLargeList do
@@ -474,6 +495,10 @@ defmodule KernelTest do
     def fun_in(x) when x in @at_range, do: :at_range
     def fun_in(x) when x in [9 | [10, 11]], do: :list_cons
     def fun_in(x) when x in [12 | @at_list2], do: :list_cons_at
+    def fun_in(x) when x in 21..15//1, do: raise("oops positive")
+    def fun_in(x) when x in 15..21//-1, do: raise("oops negative")
+    def fun_in(x) when x in 15..21//2, do: :range_step_2
+    def fun_in(x) when x in 15..21//1, do: :range_step_1
     def fun_in(_), do: :none
 
     test "in function guard" do
@@ -491,6 +516,10 @@ defmodule KernelTest do
       assert fun_in(12) == :list_cons_at
       assert fun_in(13) == :list_cons_at
       assert fun_in(14) == :list_cons_at
+      assert fun_in(15) == :range_step_2
+      assert fun_in(16) == :range_step_1
+      assert fun_in(17) == :range_step_2
+      assert fun_in(22) == :none
 
       assert fun_in(0.0) == :none
       assert fun_in(1.0) == :none
@@ -505,12 +534,15 @@ defmodule KernelTest do
       assert fun_in(12.0) == :none
       assert fun_in(13.0) == :none
       assert fun_in(14.0) == :none
+      assert fun_in(15.0) == :none
+      assert fun_in(16.0) == :none
+      assert fun_in(17.0) == :none
     end
 
     def dynamic_in(x, y, z) when x in y..z, do: true
     def dynamic_in(_x, _y, _z), do: false
 
-    test "in dynamic function guard" do
+    test "in dynamic range function guard" do
       assert dynamic_in(1, 1, 3)
       assert dynamic_in(2, 1, 3)
       assert dynamic_in(3, 1, 3)
@@ -529,6 +561,33 @@ defmodule KernelTest do
       refute dynamic_in(2.0, 1, 3)
     end
 
+    def dynamic_step_in(x, y, z, w) when x in y..z//w, do: true
+    def dynamic_step_in(_x, _y, _z, _w), do: false
+
+    test "in dynamic range with step function guard" do
+      assert dynamic_step_in(1, 1, 3, 1)
+      assert dynamic_step_in(2, 1, 3, 1)
+      assert dynamic_step_in(3, 1, 3, 1)
+
+      refute dynamic_step_in(1, 1, 3, -1)
+      refute dynamic_step_in(2, 1, 3, -1)
+      refute dynamic_step_in(3, 1, 3, -1)
+
+      assert dynamic_step_in(1, 3, 1, -1)
+      assert dynamic_step_in(2, 3, 1, -1)
+      assert dynamic_step_in(3, 3, 1, -1)
+
+      refute dynamic_step_in(1, 3, 1, 1)
+      refute dynamic_step_in(2, 3, 1, 1)
+      refute dynamic_step_in(3, 3, 1, 1)
+
+      assert dynamic_step_in(1, 1, 3, 2)
+      refute dynamic_step_in(2, 1, 3, 2)
+      assert dynamic_step_in(3, 1, 3, 2)
+      assert dynamic_step_in(3, 1, 4, 2)
+      refute dynamic_step_in(4, 1, 4, 2)
+    end
+
     defmacrop case_in(x, y) do
       quote do
         case 0 do
@@ -554,24 +613,6 @@ defmodule KernelTest do
       refute map_dot(%{})
       refute map_dot(%{field: false})
       assert map_dot(%{field: true})
-
-      message =
-        "cannot invoke remote function in guard. " <>
-          "If you want to do a map lookup instead, please remove parens from map.field()"
-
-      assert_raise CompileError, Regex.compile!(message), fn ->
-        defmodule MapDot do
-          def map_dot(map) when map.field(), do: true
-        end
-      end
-
-      message = ~r"cannot invoke remote function Module.fun/0 inside guards"
-
-      assert_raise CompileError, message, fn ->
-        defmodule MapDot do
-          def map_dot(map) when Module.fun(), do: true
-        end
-      end
     end
 
     test "performs all side-effects" do
@@ -615,7 +656,7 @@ defmodule KernelTest do
     end
 
     test "with a non-literal non-escaped compile-time range in guards" do
-      message = "non-literal range in guard should be escaped with Macro.escape/2"
+      message = ~r"found unescaped value on the right side of in/2: 1..3"
 
       assert_eval_raise(ArgumentError, message, """
       defmodule InErrors do
@@ -670,18 +711,16 @@ defmodule KernelTest do
       result = expand_to_string(quote(do: rand() in 1..2))
       assert result =~ "var = rand()"
 
-      assert result =~
-               ":erlang.andalso(:erlang.is_integer(var), :erlang.andalso(:erlang.>=(var, 1), :erlang.\"=<\"(var, 2)))"
+      assert result =~ """
+             :erlang.andalso(
+               :erlang.is_integer(var),
+               :erlang.andalso(:erlang.>=(var, 1), :erlang.\"=<\"(var, 2))
+             )\
+             """
 
       # Empty list
-      assert expand_to_string(quote(do: :x in [])) =~
-               ~S"""
-                 _ = :x
-                 false
-               """
-
-      assert expand_to_string(quote(do: :x in []), :guard) ==
-               "false"
+      assert expand_to_string(quote(do: :x in [])) =~ "_ = :x\nfalse"
+      assert expand_to_string(quote(do: :x in []), :guard) == "false"
 
       # Lists
       result = expand_to_string(quote(do: rand() in [1, 2]))
@@ -689,30 +728,10 @@ defmodule KernelTest do
       assert result =~ ":erlang.orelse(:erlang.\"=:=\"(var, 1), :erlang.\"=:=\"(var, 2))"
 
       result = expand_to_string(quote(do: rand() in [1 | [2]]))
-      assert result =~ "var = rand()"
-      assert result =~ ":erlang.orelse(:erlang.\"=:=\"(var, 1), :erlang.\"=:=\"(var, 2))"
-
-      result = expand_to_string(quote(do: rand() in [1, 2 | [3]]))
-      assert result =~ "var = rand()"
-
-      assert result =~
-               ":erlang.orelse(:erlang.\"=:=\"(var, 1), :erlang.orelse(:erlang.\"=:=\"(var, 2), :erlang.\"=:=\"(var, 3)))"
-
-      result = expand_to_string(quote(do: rand() in [1 | [2 | [3]]]))
-      assert result =~ "var = rand()"
-
-      assert result =~
-               ":erlang.orelse(:erlang.\"=:=\"(var, 1), :erlang.orelse(:erlang.\"=:=\"(var, 2), :erlang.\"=:=\"(var, 3)))"
+      assert result =~ ":lists.member(rand(), [1 | [2]]"
 
       result = expand_to_string(quote(do: rand() in [1 | some_call()]))
-      assert result =~ "var = rand()"
-      assert result =~ "{arg0} = {some_call()}"
-      assert result =~ ":erlang.orelse(:erlang.\"=:=\"(var, 1), :lists.member(var, arg0))"
-
-      result = expand_to_string(quote(do: rand() in [{1}, {2}, {3} | some_call()]))
-      assert result =~ "var = rand()"
-      assert result =~ "{arg0, arg1, arg2, arg3} = {{1}, {2}, {3}, some_call()}"
-      assert result =~ ":erlang.orelse(:erlang.\"=:=\"(var, arg2), :lists.member(var, arg3)))"
+      assert result =~ ":lists.member(rand(), [1 | some_call()]"
     end
 
     defp quote_case_in(left, right) do
@@ -724,20 +743,29 @@ defmodule KernelTest do
     end
 
     test "is optimized" do
-      assert expand_to_string(quote(do: foo in [])) =~
-               ~S"""
-                 _ = foo
-                 false
-               """
+      assert expand_to_string(quote(do: foo in [])) ==
+               "_ = foo\nfalse"
 
-      assert expand_to_string(quote(do: foo in [foo])) =~
-               ~r/{arg0} = {foo}\n\s+:erlang."=:="\(foo, arg0\)\n/
+      assert expand_to_string(quote(do: foo in [1, 2, 3])) == """
+             :erlang.orelse(
+               :erlang.orelse(:erlang.\"=:=\"(foo, 1), :erlang.\"=:=\"(foo, 2)),
+               :erlang.\"=:=\"(foo, 3)
+             )\
+             """
 
-      assert expand_to_string(quote(do: foo in 0..1)) ==
-               ":erlang.andalso(:erlang.is_integer(foo), :erlang.andalso(:erlang.>=(foo, 0), :erlang.\"=<\"(foo, 1)))"
+      assert expand_to_string(quote(do: foo in 0..1)) == """
+             :erlang.andalso(
+               :erlang.is_integer(foo),
+               :erlang.andalso(:erlang.>=(foo, 0), :erlang.\"=<\"(foo, 1))
+             )\
+             """
 
-      assert expand_to_string(quote(do: foo in -1..0)) ==
-               ":erlang.andalso(:erlang.is_integer(foo), :erlang.andalso(:erlang.>=(foo, -1), :erlang.\"=<\"(foo, 0)))"
+      assert expand_to_string(quote(do: foo in -1..0)) == """
+             :erlang.andalso(
+               :erlang.is_integer(foo),
+               :erlang.andalso(:erlang.>=(foo, -1), :erlang.\"=<\"(foo, 0))
+             )\
+             """
 
       assert expand_to_string(quote(do: foo in 1..1)) ==
                ":erlang.\"=:=\"(foo, 1)"
@@ -765,6 +793,11 @@ defmodule KernelTest do
       refute {:__info__, 1} in Kernel.__info__(:functions)
     end
 
+    test ":struct" do
+      assert Kernel.__info__(:struct) == nil
+      assert hd(URI.__info__(:struct)) == %{field: :scheme, required: false}
+    end
+
     test "others" do
       assert Kernel.__info__(:module) == Kernel
       assert is_list(Kernel.__info__(:compile))
@@ -798,11 +831,25 @@ defmodule KernelTest do
     end
 
     test "matching attribute" do
-      assert_raise ArgumentError, ~r"invalid write attribute syntax", fn ->
+      assert_raise ArgumentError, ~r"invalid usage of module attributes", fn ->
         defmodule MatchAttributeInModule do
           @foo = 42
         end
       end
+
+      assert_raise ArgumentError, ~r"invalid usage of module attributes", fn ->
+        defmodule MatchAttributeInModule do
+          @foo 16
+          <<_::@foo>> = "ab"
+        end
+      end
+
+      assert_raise ArgumentError, ~r"invalid usage of module attributes", fn ->
+        defmodule MatchAttributeInModule do
+          @foo 16
+          <<_::size(@foo)>> = "ab"
+        end
+      end
     end
   end
 
@@ -821,6 +868,18 @@ defmodule KernelTest do
     end
 
     test "raises with non-variable arguments" do
+      assert_raise ArgumentError,
+                   "guards are not allowed in defdelegate/2, got: when is_list(term) or is_binary(term)",
+                   fn ->
+                     string = """
+                     defmodule IntDelegateWithGuards do
+                       defdelegate foo(term) when is_list(term) or is_binary(term), to: List
+                     end
+                     """
+
+                     Code.eval_string(string, [], __ENV__)
+                   end
+
       msg = "defdelegate/2 only accepts function parameters, got: 1"
 
       assert_raise ArgumentError, msg, fn ->
@@ -844,6 +903,16 @@ defmodule KernelTest do
       end
     end
 
+    test "raises when :to targeting the delegating module is given without the :as option" do
+      assert_raise ArgumentError,
+                   ~r/defdelegate function is calling itself, which will lead to an infinite loop. You should either change the value of the :to option or specify the :as option/,
+                   fn ->
+                     defmodule ImplAttributes do
+                       defdelegate foo(), to: __MODULE__
+                     end
+                   end
+    end
+
     defdelegate my_reverse(list \\ []), to: :lists, as: :reverse
     defdelegate my_get(map \\ %{}, key, default \\ ""), to: Map, as: :get
 
@@ -859,17 +928,35 @@ defmodule KernelTest do
   end
 
   describe "defmodule" do
+    test "expects atoms as module names" do
+      msg = ~r"invalid module name: 3"
+
+      assert_raise ArgumentError, msg, fn ->
+        defmodule 1 + 2, do: :ok
+      end
+    end
+
     test "does not accept special atoms as module names" do
       special_atoms = [nil, true, false]
 
       Enum.each(special_atoms, fn special_atom ->
         msg = ~r"invalid module name: #{inspect(special_atom)}"
 
-        assert_raise CompileError, msg, fn ->
+        assert_raise ArgumentError, msg, fn ->
           defmodule special_atom, do: :ok
         end
       end)
     end
+
+    test "does not accept slashes in module names" do
+      assert_raise ArgumentError, ~r(invalid module name: :"foo/bar"), fn ->
+        defmodule :"foo/bar", do: :ok
+      end
+
+      assert_raise ArgumentError, ~r(invalid module name: :"foo\\\\bar"), fn ->
+        defmodule :"foo\\bar", do: :ok
+      end
+    end
   end
 
   describe "access" do
@@ -886,7 +973,7 @@ defmodule KernelTest do
       map = %{"fruits" => ["banana", "apple", "orange"]}
       assert get_in(map, ["fruits", by_index(0)]) == "banana"
       assert get_in(map, ["fruits", by_index(3)]) == nil
-      assert get_in(map, ["unknown", by_index(3)]) == :oops
+      assert get_in(map, ["unknown", by_index(3)]) == nil
 
       assert_raise FunctionClauseError, fn ->
         get_in(users, [])
@@ -1100,12 +1187,15 @@ defmodule KernelTest do
 
     def by_index(index) do
       fn
-        _, nil, next ->
-          next.(:oops)
+        :get, nil, _next ->
+          raise "won't be invoked"
 
         :get, data, next ->
           next.(Enum.at(data, index))
 
+        :get_and_update, nil, next ->
+          next.(:oops)
+
         :get_and_update, data, next ->
           current = Enum.at(data, index)
 
@@ -1139,6 +1229,10 @@ defmodule KernelTest do
       assert [1] |> (&hd(&1)).() == 1
     end
 
+    test "reverse associativity" do
+      assert [1, [2], 3] |> (List.flatten() |> Enum.map(&(&1 * 2))) == [2, 4, 6]
+    end
+
     defp twice(a), do: a * 2
 
     defp local(list) do
@@ -1277,13 +1371,20 @@ defmodule KernelTest do
     end
   end
 
+  test "tap/1" do
+    import ExUnit.CaptureIO
+
+    assert capture_io(fn ->
+             tap("foo", &IO.puts/1)
+           end) == "foo\n"
+
+    assert 1 = tap(1, fn x -> x + 1 end)
+  end
+
   test "tl/1" do
     assert tl([:one]) == []
     assert tl([1, 2, 3]) == [2, 3]
-
-    assert_raise ArgumentError, "argument error", fn ->
-      tl(empty_list())
-    end
+    assert_raise ArgumentError, fn -> tl(empty_list()) end
 
     assert tl([:a | :b]) == :b
     assert tl([:a, :b | :c]) == [:b | :c]
@@ -1291,11 +1392,7 @@ defmodule KernelTest do
 
   test "hd/1" do
     assert hd([1, 2, 3, 4]) == 1
-
-    assert_raise ArgumentError, "argument error", fn ->
-      hd(empty_list())
-    end
-
+    assert_raise ArgumentError, fn -> hd(empty_list()) end
     assert hd([1 | 2]) == 1
   end
 
@@ -1327,6 +1424,13 @@ defmodule KernelTest do
     assert match?(x when ceil(x) == 1, 0.2)
   end
 
+  test "binary_slice/2" do
+    assert binary_slice("abc", -1..0) == ""
+    assert binary_slice("abc", -5..-5) == ""
+    assert binary_slice("x", 0..0//2) == "x"
+    assert binary_slice("abcde", 1..3//2) == "bd"
+  end
+
   test "sigil_U/2" do
     assert ~U[2015-01-13 13:00:07.123Z] == %DateTime{
              calendar: Calendar.ISO,
@@ -1347,6 +1451,10 @@ defmodule KernelTest do
       Code.eval_string(~s{~U[2015-01-13 13:00]})
     end
 
+    assert_raise ArgumentError, ~r"reason: :invalid_format", fn ->
+      Code.eval_string(~s{~U[20150113 130007Z]})
+    end
+
     assert_raise ArgumentError, ~r"reason: :missing_offset", fn ->
       Code.eval_string(~s{~U[2015-01-13 13:00:07]})
     end
@@ -1355,4 +1463,60 @@ defmodule KernelTest do
       Code.eval_string(~s{~U[2015-01-13 13:00:07+00:30]})
     end
   end
+
+  describe "dbg/2" do
+    import ExUnit.CaptureIO
+
+    test "prints the given expression and returns its value" do
+      output = capture_io(fn -> assert dbg(List.duplicate(:foo, 3)) == [:foo, :foo, :foo] end)
+      assert output =~ "kernel_test.exs"
+      assert output =~ "KernelTest"
+      assert output =~ "List"
+      assert output =~ "duplicate"
+      assert output =~ ":foo"
+      assert output =~ "3"
+    end
+
+    test "doesn't print any colors if :syntax_colors is []" do
+      output =
+        capture_io(fn ->
+          assert dbg(List.duplicate(:foo, 3), syntax_colors: []) == [:foo, :foo, :foo]
+        end)
+
+      assert output =~ "kernel_test.exs"
+      assert output =~ "KernelTest."
+      assert output =~ "List.duplicate(:foo, 3)"
+      assert output =~ "[:foo, :foo, :foo]"
+      refute output =~ "\\e["
+    end
+
+    test "prints binding() if no arguments are given" do
+      my_var = 1
+      my_other_var = :foo
+
+      output = capture_io(fn -> dbg() end)
+
+      assert output =~ "binding()"
+      assert output =~ "my_var:"
+      assert output =~ "my_other_var:"
+    end
+
+    test "is not allowed in guards" do
+      message = "invalid expression in guard, dbg is not allowed in guards"
+
+      assert_raise ArgumentError, Regex.compile!(message), fn ->
+        defmodule DbgGuard do
+          def dbg_guard() when dbg(1), do: true
+        end
+      end
+    end
+
+    test "is not allowed in pattern matches" do
+      message = "invalid expression in match, dbg is not allowed in patterns"
+
+      assert_eval_raise(ArgumentError, Regex.compile!(message), """
+      {:ok, dbg()} = make_ref()
+      """)
+    end
+  end
 end
