@@ -2,7 +2,7 @@ Code.require_file("../../test_helper.exs", __DIR__)
 
 defmodule TypeHelper do
   alias Module.Types
-  alias Module.Types.{Pattern, Expr}
+  alias Module.Types.{Pattern, Expr, Unify}
 
   defmacro quoted_expr(patterns \\ [], guards \\ [], body) do
     expr = expand_expr(patterns, guards, body, __CALLER__)
@@ -15,8 +15,9 @@ defmodule TypeHelper do
   def __expr__({patterns, guards, body}) do
     with {:ok, _types, context} <-
            Pattern.of_head(patterns, guards, new_stack(), new_context()),
-         {:ok, type, context} <- Expr.of_expr(body, new_stack(), context) do
-      {:ok, Types.lift_type(type, context)}
+         {:ok, type, context} <- Expr.of_expr(body, :dynamic, new_stack(), context) do
+      {[type], _context} = Unify.lift_types([type], context)
+      {:ok, type}
     else
       {:error, {type, reason, _context}} ->
         {:error, {type, reason}}
@@ -29,7 +30,7 @@ defmodule TypeHelper do
         fn unquote(patterns) when unquote(guards) -> unquote(expr) end
       end
 
-    {ast, _env} = :elixir_expand.expand(fun, env)
+    {ast, _, _} = :elixir_expand.expand(fun, :elixir_env.env_to_ex(env), env)
     {:fn, _, [{:->, _, [[{:when, _, [patterns, guards]}], body]}]} = ast
     {patterns, guards, body}
   end
