@@ -121,7 +121,7 @@ defmodule ModuleTest do
   end
 
   test "in memory modules are tagged as so" do
-    assert :code.which(__MODULE__) == ''
+    assert :code.which(__MODULE__) == ~c""
   end
 
   ## Eval
@@ -231,7 +231,7 @@ defmodule ModuleTest do
       @compile {:autoload, false}
     end
 
-    refute :code.is_loaded(NoAutoload)
+    refute Code.loaded?(NoAutoload)
   end
 
   ## Naming
@@ -442,6 +442,13 @@ defmodule ModuleTest do
     assert backend.debug_info(:elixir_v1, ModuleCreateNoDebugInfo, data, []) == {:error, :missing}
   end
 
+  test "compiles to core" do
+    {:ok, {Atom, [{~c"Dbgi", dbgi}]}} = Atom |> :code.which() |> :beam_lib.chunks([~c"Dbgi"])
+    {:debug_info_v1, backend, data} = :erlang.binary_to_term(dbgi)
+    {:ok, core} = backend.debug_info(:core_v1, Atom, data, [])
+    assert is_tuple(core)
+  end
+
   test "no function in module body" do
     in_module do
       assert __ENV__.function == nil
@@ -559,6 +566,8 @@ defmodule ModuleTest do
         Module.put_attribute(__MODULE__, :attribute, 1)
         assert Module.get_attribute(__MODULE__, :attribute) == 1
         assert Module.get_attribute(__MODULE__, :attribute, :default) == 1
+        Module.put_attribute(__MODULE__, :attribute, nil)
+        assert Module.get_attribute(__MODULE__, :attribute, :default) == nil
       end
     end
 
@@ -569,6 +578,41 @@ defmodule ModuleTest do
     end
   end
 
+  describe "get_last_attribute/3" do
+    test "returns the last set value when the attribute is marked as `accumulate: true`" do
+      in_module do
+        Module.register_attribute(__MODULE__, :value, accumulate: true)
+        Module.put_attribute(__MODULE__, :value, 1)
+        assert Module.get_last_attribute(__MODULE__, :value) == 1
+        Module.put_attribute(__MODULE__, :value, 2)
+        assert Module.get_last_attribute(__MODULE__, :value) == 2
+      end
+    end
+
+    test "returns the value of the non-accumulate attribute if it exists" do
+      in_module do
+        Module.put_attribute(__MODULE__, :attribute, 1)
+        assert Module.get_last_attribute(__MODULE__, :attribute) == 1
+        Module.put_attribute(__MODULE__, :attribute, nil)
+        assert Module.get_last_attribute(__MODULE__, :attribute, :default) == nil
+      end
+    end
+
+    test "returns the passed default if the accumulate attribute has not yet been set" do
+      in_module do
+        Module.register_attribute(__MODULE__, :value, accumulate: true)
+        assert Module.get_last_attribute(__MODULE__, :value) == nil
+        assert Module.get_last_attribute(__MODULE__, :value, :default) == :default
+      end
+    end
+
+    test "returns the passed default if the non-accumulate attribute does not exist" do
+      in_module do
+        assert Module.get_last_attribute(__MODULE__, :value, :default) == :default
+      end
+    end
+  end
+
   describe "has_attribute?/2 and attributes_in/2" do
     test "returns true when attribute has been defined" do
       in_module do
