@@ -23,11 +23,11 @@ defmodule ExUnit.Case do
     * `:register` - when `false`, does not register this module within
       ExUnit server. This means the module won't run when ExUnit suite runs.
 
-  This module automatically includes all callbacks defined in
-  `ExUnit.Callbacks`. See that module for more information on `setup`,
-  `start_supervised`, `on_exit` and the test process life cycle.
-
-  For grouping tests together, see `describe/2` in this module.
+  > #### `use ExUnit.Case` {: .info}
+  >
+  > When you `use ExUnit.Case`, it will import the functionality
+  > from `ExUnit.Assertions`, `ExUnit.Callbacks`, `ExUnit.DocTest`,
+  > and this module itself.
 
   ## Examples
 
@@ -138,6 +138,8 @@ defmodule ExUnit.Case do
   If the same key is set via `@tag`, the `@tag` value has higher
   precedence.
 
+  The `setup_all` blocks only receive tags that are set using `@moduletag`.
+
   ### Known tags
 
   The following tags are set automatically by ExUnit and are
@@ -163,6 +165,11 @@ defmodule ExUnit.Case do
 
     * `:doctest_line` - the line the doctest was defined (if a doctest)
 
+    * `:doctest_data` - additional metadata about doctests (if a doctest)
+
+    * `:test_type` - the test type used when printing test results.
+      It is set by ExUnit to `:test`, `:doctest` and so on, but is customizable.
+
   The following tags customize how tests behave:
 
     * `:capture_log` - see the "Log Capture" section below
@@ -174,9 +181,6 @@ defmodule ExUnit.Case do
 
     * `:tmp_dir` - (since v1.11.0) see the "Tmp Dir" section below
 
-    * `:test_type` - the test type used when printing test results.
-      It is set by ExUnit to `:test`, `:doctest` and so on, but is customizable.
-
   ## Filters
 
   Tags can also be used to identify specific tests, which can then
@@ -191,7 +195,7 @@ defmodule ExUnit.Case do
   set to `true`. This behaviour can be reversed with the `:include` option
   which is usually passed through the command line:
 
-      mix test --include external:true
+      $ mix test --include external:true
 
   Run `mix help test` for more information on how to run filters via Mix.
 
@@ -427,11 +431,11 @@ defmodule ExUnit.Case do
 
   When using Mix, you can run all tests in a describe block by name:
 
-      mix test --only describe:"String.capitalize/1"
+      $ mix test --only describe:"String.capitalize/1"
 
   or by passing the exact line the describe block starts on:
 
-      mix test path/to/file:123
+      $ mix test path/to/file:123
 
   Note describe blocks cannot be nested. Instead of relying on hierarchy
   for composition, developers should build on top of named setups. For
@@ -461,6 +465,7 @@ defmodule ExUnit.Case do
   for the developer to glance at each describe block and know exactly the
   setup steps involved.
   """
+  @doc since: "1.3.0"
   defmacro describe(message, do: block) do
     definition =
       quote unquote: false do
@@ -490,9 +495,23 @@ defmodule ExUnit.Case do
       |> Enum.reverse()
       |> Macro.escape()
 
+    moduletag = Module.get_attribute(env.module, :moduletag)
+
+    tags =
+      moduletag
+      |> normalize_tags()
+      |> validate_tags()
+      |> Map.new()
+      |> Map.merge(%{module: env.module, case: env.module})
+
     quote do
       def __ex_unit__ do
-        %ExUnit.TestModule{file: __ENV__.file, name: __MODULE__, tests: unquote(tests)}
+        %ExUnit.TestModule{
+          file: __ENV__.file,
+          name: __MODULE__,
+          tags: unquote(Macro.escape(tags)),
+          tests: unquote(tests)
+        }
       end
     end
   end
@@ -526,6 +545,7 @@ defmodule ExUnit.Case do
   display. You can use `ExUnit.plural_rule/2` to set a custom
   pluralization.
   """
+  @doc since: "1.10.0"
   def register_test(mod, file, line, test_type, name, tags) do
     unless Module.has_attribute?(mod, :ex_unit_tests) do
       raise "cannot define #{test_type}. Please make sure you have invoked " <>
@@ -596,6 +616,7 @@ defmodule ExUnit.Case do
   """
   # TODO: Deprecate on Elixir v1.17
   @doc deprecated: "Use register_test/6 instead"
+  @doc since: "1.3.0"
   def register_test(%{module: mod, file: file, line: line}, test_type, name, tags) do
     register_test(mod, file, line, test_type, name, tags)
   end
@@ -628,6 +649,7 @@ defmodule ExUnit.Case do
       end
 
   """
+  @doc since: "1.3.0"
   @spec register_attribute(env, atom, keyword) :: :ok
   def register_attribute(env, name, opts \\ [])
   def register_attribute(%{module: mod}, name, opts), do: register_attribute(mod, name, opts)
@@ -745,6 +767,27 @@ defmodule ExUnit.Case do
     end
   end
 
+  @doc """
+  Returns the most recently registered test case as an `%ExUnit.Test{}`
+  struct.
+
+  This is used by third-party utilities to allow compile-time configuration
+  using test tags without having to explicitly pass the test context at
+  run-time. It is intended to be invoked in macros before the test module
+  is compiled.
+
+  Raises if called with a module that has already been compiled.
+  """
+  @doc since: "1.15.0"
+  @spec get_last_registered_test(env) :: ExUnit.Test.t() | nil
+  def get_last_registered_test(%{module: mod}) do
+    get_last_registered_test(mod)
+  end
+
+  def get_last_registered_test(mod) when is_atom(mod) do
+    Module.get_last_attribute(mod, :ex_unit_tests)
+  end
+
   defp validate_tags(tags) do
     for tag <- @reserved, Map.has_key?(tags, tag) do
       raise "cannot set tag #{inspect(tag)} because it is reserved by ExUnit"
