@@ -98,19 +98,11 @@ defmodule IEx.Helpers do
       consolidation = Mix.Project.consolidation_path(config)
       reenable_tasks(config)
 
-      # No longer allow consolidations to be accessed.
-      Code.delete_path(consolidation)
-      purge_protocols(consolidation)
-
       force? = Keyword.get(options, :force, false)
-      arguments = if force?, do: ["--force"], else: []
-
-      {result, _} = Mix.Task.run("compile", arguments)
-
-      # Re-enable consolidation and allow them to be loaded.
-      Code.prepend_path(consolidation)
-      purge_protocols(consolidation)
+      args = ["--purge-consolidation-path-if-stale", "--return-errors", consolidation]
+      args = if force?, do: ["--force" | args], else: args
 
+      {result, _} = Mix.Task.run("compile", args)
       result
     else
       IO.puts(IEx.color(:eval_error, "Mix is not running. Please start IEx with: iex -S mix"))
@@ -130,20 +122,6 @@ defmodule IEx.Helpers do
     Enum.each(compilers, &Mix.Task.reenable("compile.#{&1}"))
   end
 
-  defp purge_protocols(path) do
-    case File.ls(path) do
-      {:ok, beams} ->
-        Enum.each(beams, fn beam ->
-          module = beam |> Path.rootname() |> String.to_atom()
-          :code.purge(module)
-          :code.delete(module)
-        end)
-
-      {:error, _} ->
-        :ok
-    end
-  end
-
   @doc """
   Compiles the given files.
 
@@ -531,7 +509,7 @@ defmodule IEx.Helpers do
     |> Enum.map_join(", ", &inspect/1)
   end
 
-  @runtime_info_topics [:system, :memory, :limits, :applications]
+  @runtime_info_topics [:system, :memory, :limits, :applications, :allocators]
   @doc """
   Prints VM/runtime information such as versions, memory usage and statistics.
 
@@ -570,9 +548,9 @@ defmodule IEx.Helpers do
   end
 
   defp print_topic_info(topics) when is_list(topics) do
-    IO.write(pad_key("Showing topics"))
+    IO.write(pad_key("Showing topics:"))
     IO.puts(inspect(topics))
-    IO.write(pad_key("Additional topics"))
+    IO.write(pad_key("Additional topics:"))
     IO.puts(inspect(@runtime_info_topics -- topics))
     IO.puts("")
     IO.puts("To view a specific topic call runtime_info(topic)")
@@ -591,12 +569,12 @@ defmodule IEx.Helpers do
 
   defp print_runtime_info_topic(:memory) do
     print_pane("Memory")
-    print_memory("Total", :total)
     print_memory("Atoms", :atom)
     print_memory("Binaries", :binary)
     print_memory("Code", :code)
     print_memory("ETS", :ets)
     print_memory("Processes", :processes)
+    print_memory("Total", :total)
   end
 
   defp print_runtime_info_topic(:limits) do
@@ -611,12 +589,12 @@ defmodule IEx.Helpers do
   end
 
   defp print_runtime_info_topic(:applications) do
-    print_pane("Loaded OTP Applications")
+    print_pane("Loaded OTP applications")
     started = Application.started_applications()
     loaded = Application.loaded_applications()
 
     for {app, _, version} = entry <- Enum.sort(loaded) do
-      IO.write(pad_key(app))
+      IO.write(pad_key(Atom.to_string(app)))
       IO.write(String.pad_trailing("#{version}", 20))
 
       if entry in started do
@@ -629,6 +607,38 @@ defmodule IEx.Helpers do
     :ok
   end
 
+  @allocator_column_padding 12
+  @allocator_size_padding 19
+
+  defp print_runtime_info_topic(:allocators) do
+    allocator_sizes_map = get_allocator_sizes()
+
+    if Enum.empty?(allocator_sizes_map) do
+      IO.puts(IEx.color(:eval_error, "Could not get allocator sizes."))
+    else
+      print_pane("Memory allocators")
+
+      IO.puts([
+        String.duplicate(" ", @allocator_column_padding),
+        String.pad_leading("Block size", @allocator_size_padding),
+        String.pad_leading("Carrier size", @allocator_size_padding),
+        String.pad_leading("Max carrier size", @allocator_size_padding)
+      ])
+
+      print_allocator("Temporary", allocator_sizes_map.temp_alloc)
+      print_allocator("Short-lived", allocator_sizes_map.sl_alloc)
+      print_allocator("STD", allocator_sizes_map.std_alloc)
+      print_allocator("Long-lived", allocator_sizes_map.ll_alloc)
+      print_allocator("Erlang heap", allocator_sizes_map.eheap_alloc)
+      print_allocator("ETS", allocator_sizes_map.ets_alloc)
+      print_allocator("Fix", allocator_sizes_map.fix_alloc)
+      print_allocator("Literal", allocator_sizes_map.literal_alloc)
+      print_allocator("Binary", allocator_sizes_map.binary_alloc)
+      print_allocator("Driver", allocator_sizes_map.driver_alloc)
+      print_allocator("Total", allocator_sizes_map.total)
+    end
+  end
+
   defp print_pane(msg) do
     IO.puts(IEx.color(:eval_result, ["\n## ", msg, " \n"]))
   end
@@ -656,6 +666,71 @@ defmodule IEx.Helpers do
     IO.puts("#{pad_key(key)}#{format_bytes(value)}")
   end
 
+  defp print_allocator(title, %{
+         block_size: block_size,
+         carrier_size: carriers_size,
+         max_carrier_size: max_carriers_size
+       }) do
+    IO.puts([
+      String.pad_trailing(title, @allocator_column_padding),
+      block_size |> format_bytes(:KB) |> String.pad_leading(@allocator_size_padding),
+      carriers_size |> format_bytes(:KB) |> String.pad_leading(@allocator_size_padding),
+      max_carriers_size |> format_bytes(:KB) |> String.pad_leading(@allocator_size_padding)
+    ])
+  end
+
+  defp get_allocator_sizes() do
+    alloc_util_allocators = :erlang.system_info(:alloc_util_allocators)
+
+    allocators_map =
+      Map.new(
+        [:total | alloc_util_allocators],
+        &{&1, %{block_size: 0, carrier_size: 0, max_carrier_size: 0}}
+      )
+
+    try do
+      for {allocator, allocator_instances} <-
+            :erlang.system_info({:allocator_sizes, alloc_util_allocators}),
+          {:instance, _, instance_info} <- allocator_instances,
+          {_, [{:blocks, blocks}, {:carriers_size, ccs, mcs, _}]} <-
+            instance_info,
+          reduce: allocators_map do
+        acc ->
+          cbs =
+            case blocks do
+              [{_, [{:size, bs, _, _}]}] -> bs
+              _ -> 0
+            end
+
+          %{
+            block_size: block_size,
+            carrier_size: carrier_size,
+            max_carrier_size: max_carrier_size
+          } = Map.fetch!(acc, allocator)
+
+          %{
+            block_size: total_block_size,
+            carrier_size: total_carrier_size,
+            max_carrier_size: total_max_carrier_size
+          } = Map.fetch!(acc, :total)
+
+          acc
+          |> Map.replace!(allocator, %{
+            block_size: block_size + cbs,
+            carrier_size: carrier_size + ccs,
+            max_carrier_size: max_carrier_size + mcs
+          })
+          |> Map.replace!(:total, %{
+            block_size: total_block_size + cbs,
+            carrier_size: total_carrier_size + ccs,
+            max_carrier_size: total_max_carrier_size + mcs
+          })
+      end
+    rescue
+      _ -> %{}
+    end
+  end
+
   defp format_bytes(bytes) when is_integer(bytes) do
     cond do
       bytes >= memory_unit(:GB) -> format_bytes(bytes, :GB)
@@ -680,7 +755,7 @@ defmodule IEx.Helpers do
   defp memory_unit(:MB), do: 1024 * 1024
   defp memory_unit(:KB), do: 1024
 
-  defp pad_key(key), do: String.pad_trailing("#{key}:", 20, " ")
+  defp pad_key(key), do: String.pad_trailing(key, 21, " ")
 
   @doc """
   Clears out all messages sent to the shell's inbox and prints them out.
@@ -1244,12 +1319,19 @@ defmodule IEx.Helpers do
       iex> pid("0.21.32")
       #PID<0.21.32>
 
+      iex> pid("#PID<0.21.32>")
+      #PID<0.21.32>
+
       iex> pid(:init)
       #PID<0.0.0>
 
   """
+  def pid("#PID<" <> string) do
+    :erlang.list_to_pid(~c"<#{string}")
+  end
+
   def pid(string) when is_binary(string) do
-    :erlang.list_to_pid('<#{string}>')
+    :erlang.list_to_pid(~c"<#{string}>")
   end
 
   def pid(name) when is_atom(name) do
@@ -1274,9 +1356,9 @@ defmodule IEx.Helpers do
   def pid(x, y, z)
       when is_integer(x) and x >= 0 and is_integer(y) and y >= 0 and is_integer(z) and z >= 0 do
     :erlang.list_to_pid(
-      '<' ++
+      ~c"<" ++
         Integer.to_charlist(x) ++
-        '.' ++ Integer.to_charlist(y) ++ '.' ++ Integer.to_charlist(z) ++ '>'
+        ~c"." ++ Integer.to_charlist(y) ++ ~c"." ++ Integer.to_charlist(z) ++ ~c">"
     )
   end
 
@@ -1291,7 +1373,7 @@ defmodule IEx.Helpers do
   """
   @doc since: "1.8.0"
   def port(string) when is_binary(string) do
-    :erlang.list_to_port('#Port<#{string}>')
+    :erlang.list_to_port(~c"#Port<#{string}>")
   end
 
   @doc """
@@ -1309,7 +1391,7 @@ defmodule IEx.Helpers do
   def port(major, minor)
       when is_integer(major) and major >= 0 and is_integer(minor) and minor >= 0 do
     :erlang.list_to_port(
-      '#Port<' ++ Integer.to_charlist(major) ++ '.' ++ Integer.to_charlist(minor) ++ '>'
+      ~c"#Port<" ++ Integer.to_charlist(major) ++ ~c"." ++ Integer.to_charlist(minor) ++ ~c">"
     )
   end
 
@@ -1324,7 +1406,7 @@ defmodule IEx.Helpers do
   """
   @doc since: "1.6.0"
   def ref(string) when is_binary(string) do
-    :erlang.list_to_ref('#Ref<#{string}>')
+    :erlang.list_to_ref(~c"#Ref<#{string}>")
   end
 
   @doc """
@@ -1341,11 +1423,11 @@ defmodule IEx.Helpers do
       when is_integer(w) and w >= 0 and is_integer(x) and x >= 0 and is_integer(y) and y >= 0 and
              is_integer(z) and z >= 0 do
     :erlang.list_to_ref(
-      '#Ref<' ++
+      ~c"#Ref<" ++
         Integer.to_charlist(w) ++
-        '.' ++
+        ~c"." ++
         Integer.to_charlist(x) ++
-        '.' ++ Integer.to_charlist(y) ++ '.' ++ Integer.to_charlist(z) ++ '>'
+        ~c"." ++ Integer.to_charlist(y) ++ ~c"." ++ Integer.to_charlist(z) ++ ~c">"
     )
   end
 
@@ -1376,15 +1458,14 @@ defmodule IEx.Helpers do
   def nl(nodes \\ Node.list(), module) when is_list(nodes) and is_atom(module) do
     case get_beam_and_path(module) do
       {bin, beam_path} ->
+        call = :erpc.multicall(nodes, :code, :load_binary, [module, beam_path, bin])
+
         results =
-          for node <- nodes do
-            case :rpc.call(node, :code, :load_binary, [module, beam_path, bin]) do
-              {:module, _} -> {node, :loaded, module}
-              {:badrpc, message} -> {node, :badrpc, message}
-              {:error, message} -> {node, :error, message}
-              unexpected -> {node, :error, unexpected}
-            end
-          end
+          Enum.zip_with(nodes, call, fn
+            node, {:ok, {:module, _}} -> {node, :loaded, module}
+            node, {:ok, {:error, reason}} -> {node, :error, reason}
+            node, {:error, {:erpc, reason}} -> {node, :badrpc, reason}
+          end)
 
         {:ok, results}
 
