@@ -2,10 +2,12 @@ defmodule Mix.SCM.Git do
   @behaviour Mix.SCM
   @moduledoc false
 
+  @impl true
   def fetchable? do
     true
   end
 
+  @impl true
   def format(opts) do
     if rev = get_opts_rev(opts) do
       "#{redact_uri(opts[:git])} - #{rev}"
@@ -14,6 +16,7 @@ defmodule Mix.SCM.Git do
     end
   end
 
+  @impl true
   def format_lock(opts) do
     case opts[:lock] do
       {:git, _, lock_rev, lock_opts} ->
@@ -30,28 +33,31 @@ defmodule Mix.SCM.Git do
     end
   end
 
+  @impl true
   def accepts_options(_app, opts) do
     opts =
       opts
       |> Keyword.put(:checkout, opts[:dest])
       |> sparse_opts()
+      |> subdir_opts()
 
     cond do
       gh = opts[:github] ->
         opts
         |> Keyword.delete(:github)
         |> Keyword.put(:git, "https://github.com/#{gh}.git")
-        |> validate_git_options
+        |> validate_git_options()
 
       opts[:git] ->
         opts
-        |> validate_git_options
+        |> validate_git_options()
 
       true ->
         nil
     end
   end
 
+  @impl true
   def checked_out?(opts) do
     # Are we inside a Git repository?
     opts[:checkout]
@@ -59,8 +65,8 @@ defmodule Mix.SCM.Git do
     |> File.regular?()
   end
 
+  @impl true
   def lock_status(opts) do
-    assert_git!()
     lock = opts[:lock]
 
     cond do
@@ -83,19 +89,21 @@ defmodule Mix.SCM.Git do
     end
   end
 
+  @impl true
   def equal?(opts1, opts2) do
     opts1[:git] == opts2[:git] and get_lock_opts(opts1) == get_lock_opts(opts2)
   end
 
+  @impl true
   def managers(_opts) do
     []
   end
 
+  @impl true
   def checkout(opts) do
-    assert_git!()
     path = opts[:checkout]
     File.rm_rf!(path)
-    File.mkdir_p!(path)
+    File.mkdir_p!(opts[:dest])
 
     File.cd!(path, fn ->
       git!(~w[-c core.hooksPath='' init --quiet])
@@ -104,8 +112,8 @@ defmodule Mix.SCM.Git do
     end)
   end
 
+  @impl true
   def update(opts) do
-    assert_git!()
     path = opts[:checkout]
     File.cd!(path, fn -> checkout(path, opts) end)
   end
@@ -144,6 +152,15 @@ defmodule Mix.SCM.Git do
     end
   end
 
+  defp subdir_opts(opts) do
+    if opts[:subdir] do
+      dest = Path.join(opts[:dest], opts[:subdir])
+      Keyword.put(opts, :dest, dest)
+    else
+      opts
+    end
+  end
+
   defp sparse_toggle(opts) do
     cond do
       sparse = opts[:sparse] ->
@@ -215,7 +232,7 @@ defmodule Mix.SCM.Git do
   defp get_lock_rev(_, _), do: nil
 
   defp get_lock_opts(opts) do
-    lock_opts = Keyword.take(opts, [:branch, :ref, :tag, :sparse])
+    lock_opts = Keyword.take(opts, [:branch, :ref, :tag, :sparse, :subdir])
 
     if opts[:submodules] do
       lock_opts ++ [submodules: true]
@@ -266,41 +283,47 @@ defmodule Mix.SCM.Git do
   defp git!(args, into \\ default_into()) do
     opts = cmd_opts(into: into, stderr_to_stdout: true)
 
-    case System.cmd("git", args, opts) do
+    try do
+      System.cmd("git", args, opts)
+    catch
+      :error, :enoent ->
+        Mix.raise(
+          "Error fetching/updating Git repository: the \"git\" " <>
+            "executable is not available in your PATH. Please install " <>
+            "Git on this machine or pass --no-deps-check if you want to " <>
+            "run a previously built application on a system without Git."
+        )
+    else
       {response, 0} ->
         response
 
-      {response, _} ->
+      {response, _} when is_binary(response) ->
         Mix.raise("Command \"git #{Enum.join(args, " ")}\" failed with reason: #{response}")
+
+      {_, _} ->
+        Mix.raise("Command \"git #{Enum.join(args, " ")}\" failed")
     end
   end
 
   defp default_into() do
     case Mix.shell() do
-      Mix.Shell.IO -> IO.stream(:stdio, :line)
+      Mix.Shell.IO -> IO.stream()
       _ -> ""
     end
   end
 
-  defp assert_git! do
-    case Mix.State.fetch(:git_available) do
-      {:ok, true} ->
-        :ok
-
-      :error ->
-        if System.find_executable("git") do
-          Mix.State.put(:git_available, true)
-        else
-          Mix.raise(
-            "Error fetching/updating Git repository: the \"git\" " <>
-              "executable is not available in your PATH. Please install " <>
-              "Git on this machine or pass --no-deps-check if you want to " <>
-              "run a previously built application on a system without Git."
-          )
-        end
+  # Attempt to set the current working directory by default.
+  # This addresses an issue changing the working directory when executing from
+  # within a secondary node since file I/O is done through the main node.
+  defp cmd_opts(opts) do
+    case File.cwd() do
+      {:ok, cwd} -> Keyword.put(opts, :cd, cwd)
+      _ -> opts
     end
   end
 
+  # Also invoked by lib/mix/test/test_helper.exs
+  @doc false
   def git_version do
     case Mix.State.fetch(:git_version) do
       {:ok, version} ->
@@ -310,7 +333,7 @@ defmodule Mix.SCM.Git do
         version =
           ["--version"]
           |> git!("")
-          |> parse_version
+          |> parse_version()
 
         Mix.State.put(:git_version, version)
         version
@@ -329,14 +352,4 @@ defmodule Mix.SCM.Git do
     {int, _} = Integer.parse(string)
     int
   end
-
-  # Attempt to set the current working directory by default.
-  # This addresses an issue changing the working directory when executing from
-  # within a slave node since file I/O is done through the master node.
-  defp cmd_opts(opts) do
-    case File.cwd() do
-      {:ok, cwd} -> Keyword.put(opts, :cd, cwd)
-      _ -> opts
-    end
-  end
 end
