@@ -19,7 +19,11 @@ defmodule Mix.Tasks.ArchiveTest do
     File.rm_rf!(tmp_path("userhome"))
     System.put_env("MIX_ARCHIVES", tmp_path("userhome/.mix/archives/"))
     Mix.Project.push(ArchiveProject)
-    :ok
+
+    on_exit(fn ->
+      Mix.Local.remove_archives()
+      System.delete_env("MIX_ARCHIVES")
+    end)
   end
 
   test "archive build" do
@@ -27,10 +31,11 @@ defmodule Mix.Tasks.ArchiveTest do
       Mix.Tasks.Archive.Build.run(["--no-elixir-version-check"])
       message = "Generated archive \"archive-0.1.0.ez\" with MIX_ENV=dev"
       assert_received {:mix_shell, :info, [^message]}
-      assert File.regular?('archive-0.1.0.ez')
+      assert File.regular?(~c"archive-0.1.0.ez")
+      assert to_charlist(Mix.Project.consolidation_path()) not in :code.get_path()
 
       assert_archive_content_default()
-      refute has_in_zip_file?('archive-0.1.0.ez', 'archive-0.1.0/priv/.dot_file')
+      refute has_in_zip_file?(~c"archive-0.1.0.ez", ~c"archive-0.1.0/priv/.dot_file")
     end)
   end
 
@@ -39,24 +44,24 @@ defmodule Mix.Tasks.ArchiveTest do
       Mix.Tasks.Archive.Build.run(["--no-elixir-version-check", "--include-dot-files"])
       message = "Generated archive \"archive-0.1.0.ez\" with MIX_ENV=dev"
       assert_received {:mix_shell, :info, [^message]}
-      assert File.regular?('archive-0.1.0.ez')
+      assert File.regular?(~c"archive-0.1.0.ez")
 
       assert_archive_content_default()
-      assert has_in_zip_file?('archive-0.1.0.ez', 'archive-0.1.0/priv/.dot_file')
+      assert has_in_zip_file?(~c"archive-0.1.0.ez", ~c"archive-0.1.0/priv/.dot_file")
     end)
   end
 
   def assert_archive_content_default() do
-    assert File.regular?('archive-0.1.0.ez')
-    assert has_in_zip_file?('archive-0.1.0.ez', 'archive-0.1.0/.elixir')
-    assert has_in_zip_file?('archive-0.1.0.ez', 'archive-0.1.0/priv/not_really_an.so')
+    assert File.regular?(~c"archive-0.1.0.ez")
+    assert has_in_zip_file?(~c"archive-0.1.0.ez", ~c"archive-0.1.0/.elixir")
+    assert has_in_zip_file?(~c"archive-0.1.0.ez", ~c"archive-0.1.0/priv/not_really_an.so")
 
     assert has_in_zip_file?(
-             'archive-0.1.0.ez',
-             'archive-0.1.0/ebin/Elixir.Mix.Tasks.Local.Sample.beam'
+             ~c"archive-0.1.0.ez",
+             ~c"archive-0.1.0/ebin/Elixir.Mix.Tasks.Local.Sample.beam"
            )
 
-    assert has_in_zip_file?('archive-0.1.0.ez', 'archive-0.1.0/ebin/archive.app')
+    assert has_in_zip_file?(~c"archive-0.1.0.ez", ~c"archive-0.1.0/ebin/archive.app")
   end
 
   test "archive install" do
@@ -131,10 +136,34 @@ defmodule Mix.Tasks.ArchiveTest do
     end)
   end
 
+  @compile {:no_warn_undefined, GitRepo.Archive}
   test "archive.install from Git" do
     in_fixture("git_repo", fn ->
+      File.mkdir_p!("config")
+
+      File.write!("config/config.exs", """
+      import Config
+      config :git_repo, :archive_config, true
+      """)
+
+      File.write!("lib/git_repo.ex", """
+      require Application
+      true = Application.compile_env!(:git_repo, :archive_config)
+
+      defmodule GitRepo.Archive do
+        def hello do
+          "World"
+        end
+      end
+      """)
+
+      System.cmd("git", ~w[init])
+      System.cmd("git", ~w[add .])
+      System.cmd("git", ~w[commit -m first-commit])
+
       send(self(), {:mix_shell_input, :yes?, true})
       Mix.Tasks.Archive.Install.run(["git", File.cwd!()])
+      assert GitRepo.Archive.hello() == "World"
 
       message = "Generated archive \"git_repo-0.1.0.ez\" with MIX_ENV=prod"
       assert_received {:mix_shell, :info, [^message]}
@@ -143,7 +172,7 @@ defmodule Mix.Tasks.ArchiveTest do
       assert File.dir?(tmp_path("userhome/.mix/archives/git_repo-0.1.0/git_repo-0.1.0/ebin"))
     end)
   after
-    purge([GitRepo, GitRepo.MixProject])
+    purge([GitRepo.Archive, GitRepo.MixProject])
   end
 
   test "archive install, update, and uninstall life-cycle" do
